#!/usr/bin/env python3
"""
Unit tester for sequence_info.py

Python version: Python 3.6.8 :: Anaconda, Inc.
Date: 03/22/2019
Dependencies:
 - unittest
 - biopython>=1.70
"""

import unittest
from primer_tk import sequence_info as seqinf

class TestSequence(unittest.TestCase):
    """
    Subclass of unittest to test Sequence Class for sequence_info.py
    """
    def setUp(self):
        """
        Generate a sequence object that is easy to test!
        """
        self.seq = seqinf.Sequence("ACTG")
        self.name = "seq name not seq"
        self.primer_dimer = seqinf.PrimerDimer(self.seq.sequence,
                                               seqinf.Sequence("TGCA").sequence, 2)

    def test_handles_lowercase(self):
        """
        Tests to ensure handles lowercase sequence strings.
        """
        input_seq = self.seq.sequence
        test_seq = seqinf.Sequence('actg').sequence
        self.assertEqual(input_seq, test_seq)

    def test_set_name(self):
        """
        Tests that name is properly changed when function is called.
        """
        self.name = "TestSeq1"
        self.assertEqual(self.name, "TestSeq1")

    def test_complement(self):
        """
        Tests to ensure that sequence complement is returned.
        """
        complement_seq = self.seq.complement()
        self.assertEqual(complement_seq, "TGAC")

    def test_reverse_complement(self):
        """
        Tests to ensure that reverse complement sequence is returned.
        """
        reverse_complement_seq = self.seq.reverse_complement()
        self.assertEqual(reverse_complement_seq, "CAGT")

    def test_reverse_sequence(self):
        """
        Tests to ensure that the sequence is properly reversed.
        """
        reverse_seq = self.seq.reverse_sequence()
        self.assertEqual(reverse_seq, "GTCA")


    def test_gc_percent(self):
        """
        Test to ensure that GC is properly calculated.
        """
        gc_float = self.seq.gc_percent()
        self.assertEqual(gc_float, 50.0)

    def test_0_gc_percent(self):
        """
        Test to ensure that 0 GC doesn't throw error (div0).
        """
        gc_float = seqinf.Sequence("AAAA").gc_percent()
        self.assertEqual(gc_float, 0.0)

    def test_pd_local(self):
        """
        Test to ensure pd_local is aligning exact sequences.
        ACTG & TGCA align at length 2, this is our test.
        Example:
        ACTG
        --||
          TGCA
        """
        local = self.primer_dimer.pd_local()
        self.assertEqual([item[2] for item in local], [2.0])

    def test_format_comp_align(self):
        """
        Test to ensure that complementary alignment is output.
        """
        easy_seq1 = seqinf.Sequence("AAAA").sequence
        easy_seq2 = seqinf.Sequence("TTTT").complement()
        test = seqinf.PrimerDimer(easy_seq1, easy_seq2, 4)
        local = test.pd_local()
        for item in local:
            self.assertTrue("TTTT" in seqinf.PrimerDimer.\
                                format_alignment_compl(item[0],
                                                       item[1],
                                                       item[2],
                                                       item[3],
                                                       item[4]))


    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
""" Main entry point for primer_tk """

import sys
import argparse

from primer_tk import genome_iterator, genome_iterator_sv, \
    primer_cross_hyb, \
    analyze_pcr_output, analyze_pcr_output_sv, \
    primer_tabix, \
    __version__

from primer_tk import core

def get_args(argv):
    """ Get arguments

    Args:
        argv (list): List of strings as argv.

    Returns:
        args (Namespace): Argparse object.
    """

    parser = argparse.ArgumentParser(prog="primer_tk")
    parser.add_argument("-v", "--version", action="version",
                        version=__version__)

    subparser = parser.add_subparsers(help="Actions")
    genome_iterator.add_iterator_subparser(subparser)
    genome_iterator_sv.add_iterator_subparser(subparser)
    primer_cross_hyb.add_pre_subparser(subparser)
    primer_cross_hyb.add_pre_sv_subparser(subparser)
    analyze_pcr_output.add_post_subparser(subparser)
    analyze_pcr_output_sv.add_post_subparser(subparser)
    primer_tabix.add_tabix_subparser(subparser)

    args = parser.parse_args(argv)

    return args

def main():
    """ Main rountine """

    args = get_args(sys.argv[1:])

    action = sys.argv[1]

    if action == "iterator":
        core.iterator(args)
    elif action == "iterator_sv":
        core.iterator_sv(args)
    elif action == "pre":
        core.pre(args)
    elif action == "pre_sv":
        core.pre_sv(args)
    elif action == "post":
        core.post(args)
    elif action == "post_sv":
        core.post_sv(args)
    elif action =="tabix":
        core.tabix(args)


main()



#!/usr/bin/env python3
"""
Unit tester for mp_class.py

Python version: Python 3.6.8 :: Anaconda, Inc.
Date: 03/20/2019
Dependencies:
 - unittest
 - pandas>=0.22.0
 - numpy>=1.16.0
"""

import unittest
import pandas as pd
import numpy as np
from primer_tk.mp_class import MissingPrimers
from primer_tk.mp_class import create_df

class TestMissingPrimers(unittest.TestCase):
    """
    Subclass of unittest to test mp_class.py
    """
    def setUp(self):
        self.mp = MissingPrimers("test/data/primer_dump_standard.txt", 1)

    def test_group_seqids(self):
        """
        Creates a list of lists, each sample info is stored in its own list
        within a list. Since there were 3 samples, the length of the list
        should be 3.
        """
        self.assertEqual(len(self.mp._MissingPrimers__group_seqids()), 3)

    def test_group_seqids_unfilled(self):
        """
        Tests to ensure that NA values have not been inserted into
        primer3 string.
        """
        no_missing_vals = self.mp._MissingPrimers__group_seqids()[1]
        self.assertFalse("NA" in no_missing_vals)

    def test_fill_empty_values(self):
        """
        Tests to ensure that NA values have been inserted into the 
        sample with missing primers.
        """
        all_params = self.mp._MissingPrimers__fill_empty_values()
        missing_params = all_params[1]
        self.assertTrue("=NA" in missing_params[-2])#-1 is empty space
                
    def test_gather_primer_info(self):
        """
        Test to ensure that if value 1 is passed, all sample names
        end with 1.
        """
        primer_info = self.mp._MissingPrimers__gather_primer_info()
        self.assertTrue(all(item[0].endswith('1') for item in primer_info))

    def test_invalid_value(self):
        """
        Tests to ensure class does not succeed if invalid value is passed.
        Value options are currently [0-4].
        """
        with self.assertRaises(SystemExit) as code_message:
            MissingPrimers("primer_dump_standard.txt", 6)
        self.assertEqual(code_message.exception.code,
                         "Improper value selected! Acceptable values: 0,1,2,3,4.")

    
    def test_len_list_output(self):
        """
        Tests to ensure that the list created for each sample contains 8 values.
        This corresponds to the number of df columns.
        """
        prim_list_1 = self.mp.samp_primer_info
        for sample_data in prim_list_1:
            self.assertEqual(len(sample_data), 8)

    def test_create_df(self):
        """
        Tests to ensure that a df is created from primer list
        and contains the proper number of rows.
        """
        prim_list_1 = self.mp.samp_primer_info
        primer_df = create_df([prim_list_1])
        self.assertEqual(len(primer_df), 3)

    def test_accept_multiple_prim_lists(self):
        """
        Tests to ensure that a mp_class.create_df() can accept multiple primer lists
        as input and generate an output df.
        """
        prim_list_1 = self.mp.samp_primer_info
        prim_list_2 = MissingPrimers("test/data/primer_dump_standard.txt", 2).samp_primer_info
        primer_df = create_df([prim_list_1, prim_list_2])
        self.assertEqual(len(primer_df), 6)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""
Dependencies required to run program:
 - python3.6+
    - pandas>=0.22.0
    - argparse
    - sequence_info
"""

import sys
import argparse
import pandas as pd
from primer_tk import sequence_info as seqinf

def add_post_subparser(subparser):
    """ Add subparser for postprocessing sv step.

    Args:
        subparser (?): Subparser object.

    Returns: None
    """
    parser = subparser.add_parser("post_sv", help='Parses info from pcr_output')
    parser.add_argument("-f", "--flank_file", dest="flank_file",
                        help="use flanking_regions file from output of genome_iterator_sv.py")
    parser.add_argument("-tp", "--total_primers", dest="total_primers",
                        help="the pre-PCR master primer file that\
                              contains all sample + primer info")
    parser.add_argument("-all", "--all_final_primers", default="all_final_primers_sv.csv",
                        help="all primers generated for targets")
    parser.add_argument("-top", "--top_final_primers", default="top_final_primers_sv.csv",
                        help="top primers generated for targets")
    parser.add_argument("-plate", "--plate_basename", default="plated_primers",
                        help="the basename of the primers in plate format ready to order.")

def fasta_parser(pcrfile):
    """
    Iterates through sudo-fasta output from in silico pcr
    and extracts all useful info related to samples.
    Args:
        pcrfile (file): file output from in silico pcr of primers.
    Returns:
        seqs (list): list of seqs from pcr output
        headers (list): list of headers from pcr output
    """
    seqs = []
    headers = []
    with open(pcrfile) as pcr_file:
        sequence = ""
        header = None
        for line in pcr_file:
            if line.startswith('>'):
                headers.append(line[1:-1])
                if header:
                    seqs.append(sequence)
                sequence = ""
                header = line[1:]
            else:
                sequence += line.rstrip()
        seqs.append(sequence)
    return seqs, headers

def amp_header_region(infile):
    """
    Extract relevant info from total_primer_list.csv to use for GC.
    Args:
        infile (file): total_list file to parse primer info
    Returns:
        header_start_stop (list): list of information to use when parsing fasta
    """
    primer_df = pd.read_csv(infile)
    header_start_stop = [[header, start, stop] for header, start, stop\
                             in zip(primer_df['Sequence ID'],
                                    primer_df['Primer Left Pos'],
                                    primer_df['Primer Right Pos'])]
    return header_start_stop

def get_gc_region(seqs, headers, positions):
    """
    Slice out the exact region of the fasta the primer amplified.
    Args:
        seqs (list): the list of sequences from flank_file
        headers (list): the list of headers from flank_file
        positions (list): list of tuples with header, p_len, p_seq
    Returns:
        sliced_seq (list): the exact amplified sequence.
    """
    sliced_seq = []
    for header, seq in zip(headers, seqs):
        for pos in positions:
            if pos[0] == header:
                sliced_seq.append((header, len(seq[pos[1]:pos[2]+1]), seq[pos[1]:pos[2]+1]))
    return sliced_seq

def calc_gc(sliced_seq):
    """
    Takes sliced seq list with header, product len, and product seq and returns
    the header, product len, and product GC%. A bit redundant but I might find
    it useful to have this exact sequence in the future.
    Args:
        sliced_seq (list): contains (header, product_len, product_seq)
    Returns:
        sample_gc (list): contains (header, product_len, product GC%)
    """
    sample_gc = []
    for header, p_len, p_seq in sliced_seq:
        sample_gc.append((header, p_len, seqinf.Sequence(p_seq).gc_percent()))
    return sample_gc

def merge_dfs(gc_calc, total_df, seqs):
    """
    Merges the product information onto the total primers sheet.
    Args:
        gc_calc (list): list containing header, product len, gc info
        total_df (pandas object): dataframe containing primer information
    Returns:
        merged_df (pandas object): dataframe containing updated primer information
    """

    gc = pd.DataFrame(gc_calc, columns=['Sequence Name', 'Product Len', 'Product GC%'])
    total = pd.read_csv(total_df)
    merged = pd.concat([total, gc], axis=1) # side-by-side merge
    merged['Chromosome'] = [seqid.split(':')[0].split('_')[2] for seqid in merged['Sequence ID']]
    merged['Position1'] = [seqid.split(':')[1].strip('_').split('-')[0] for seqid in merged['Sequence ID']]
    merged['Position2'] = [seqid.split(':')[1].strip('_').strip('__BP1').strip('__BP2').\
                           split('-')[1] for seqid in merged['Sequence ID']]
    seqlen = []
    for seq in seqs:
        seqlen.append(len(seq[:-1]))
    flanksize = (seqlen[0]/2)
    merged['FlankSize'] = int(flanksize)
    merged['NucsBeforeBP'] = (merged['Position1']).apply(int)\
        - ((merged['Position1'].apply(int) - merged['FlankSize'])\
        + (merged['Primer Left Pos']).apply(int))
    merged['NucsAfterBP'] = merged['Product Len'].apply(int) -\
        (merged['FlankSize'] - merged['Primer Left Pos'].apply(int))
    merged['Position1'] = ((merged['Position1'].apply(int) - merged['FlankSize'])\
                               + merged['Primer Left Pos'].apply(int))
    merged['Position2'] = (merged['Position2'].apply(int) + merged['NucsAfterBP'].apply(int))
    merged['FwdPrimPos'] = (merged['Position1']).apply(str) + '-'\
        + (merged['Position1'] + merged['Primer Left Len']).apply(str)
    merged['RvsPrimPos'] = (merged['Position2'] - merged['Primer Right Len']).apply(str)\
        + '-' + (merged['Position2']).apply(str) 
    merged_df = merged[['Sequence ID', 'Primer Rank', 'Primer Left Seq', 'Primer Right Seq',
                        'Primer Left Len', 'Primer Right Len', 'Primer Left TM',
                        'Primer Right TM', 'Primer Left GC %', 'Primer Right GC %',
                        'Product Len', 'Product GC%', 'Chromosome', 'Position1', 'Position2',
                        'NucsBeforeBP', 'NucsAfterBP', 'FwdPrimPos', 'RvsPrimPos']]
    return merged_df

def to_order_plate(top_ranking_final_primers):
    """
    Takes output from top_ranked_final_primers and organizes to easy order plates
    (Forward and Reverse).
    Args:
        top_ranked_final_primers (DataFrame): filtered, top ranked dataframe primers
    Returns:
        idt_order_sheet_plate_f (DataFrame): 96 well plate order format forward primers
        idt_order_sheet_plate_r (DataFrame): 96 well plate order format reverse primers
    """
    # Generate well numbers
    if len(top_ranking_final_primers) == 0:
        sys.exit("No primers were kept for any target. Try starting over with relaxed parameters.")
    well_and_nums = []
    wells = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
    num_wells = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12']
    for well in wells:
        for num in num_wells:
            well_and_nums.append(well+num)

    # this is required because sometimes more than 96 primers are generated and the list is consumed.
    big_well_nums = well_and_nums * 50
    top_ranking_final_primers.set_index('Sequence ID', inplace=True)
    plate_order_sheet_f = pd.DataFrame(columns=['Sequence Name', 'Sequence'])
    plate_order_sheet_r = pd.DataFrame(columns=['Sequence Name', 'Sequence'])
    names = top_ranking_final_primers.index.tolist()
    f_seq = top_ranking_final_primers['Primer Left Seq'].tolist()
    r_seq = top_ranking_final_primers['Primer Right Seq'].tolist()
    plate_order_sheet_f['Sequence Name'] = names
    plate_order_sheet_f['Sequence Name'] = plate_order_sheet_f['Sequence Name'] + '_F'
    plate_order_sheet_f['Sequence'] = f_seq
    plate_order_sheet_f.insert(0, 'Well Position', big_well_nums[:len(plate_order_sheet_f['Sequence'])])
    plate_order_sheet_r['Sequence Name'] = names
    plate_order_sheet_r['Sequence Name'] = plate_order_sheet_r['Sequence Name'] + '_R'
    plate_order_sheet_r['Sequence'] = r_seq
    plate_order_sheet_r.insert(0, 'Well Position', big_well_nums[:len(plate_order_sheet_r['Sequence'])])
    return plate_order_sheet_f, plate_order_sheet_r




#!/usr/bin/env python3

"""
Module required for program.
 - python3.6+
    - pandas >= 0.22.0
"""

import unittest
from primer_tk import analyze_pcr_output_sv as ap

class TestAnalyzePcrOutputSV(unittest.TestCase):
    """
    Subclass of unittest to test analyze_pcr_output_sv.py
    """
    def setUp(self):
        self.flanking = 'test/data/flanking_regions_sv.fasta'
        self.primer_file = 'test/data/total_list_sv.csv'
        self.seqs, self.headers = ap.fasta_parser(self.flanking)
        self.positions = ap.amp_header_region(self.primer_file)

    def test_fasta_parser(self):
        """
        Ensure that seq and header are appropriately parsed.
        """
        self.assertEqual(len(self.seqs), 3)
        self.assertEqual(len(self.headers), 3)

    def test_amp_header_region(self):
        """
        Extract header region from primer info file.
        """
        self.assertTrue(all(len(item) == 3 for item in self.positions))

    def test_get_gc_region(self):
        """
        Extract gc actual sequence region between primers based on positions.
        """
        sliced_seqs = ap.get_gc_region(self.seqs, self.headers, self.positions)
        self.assertTrue(all(len(item[2]) < 401 for item in sliced_seqs))
    def test_calc_gc(self):
        """
        Calculate the sample GC content for each sliced sequence.
        """
        sliced_seqs = ap.get_gc_region(self.seqs, self.headers, self.positions)
        gc_calc = ap.calc_gc(sliced_seqs)
        self.assertTrue(all(isinstance(item[2], float) for item in gc_calc))

    def test_merge_dfs(self):
        """
        Merges calc_gc into the original dataframe for more
        information about product length and gc content.
        """
        sliced_seqs = ap.get_gc_region(self.seqs, self.headers, self.positions)
        gc_calc = ap.calc_gc(sliced_seqs)
        self.assertEqual(len(ap.merge_dfs(gc_calc, self.primer_file, self.seqs).columns), 19)
        self.assertEqual(len(ap.merge_dfs(gc_calc, self.primer_file, self.seqs)['Sequence ID']), 13)

    def test_to_order_plate(self):
        """
        Generates a plate order format for generated primers.
        """
        sliced_seqs = ap.get_gc_region(self.seqs, self.headers, self.positions)
        gc_calc = ap.calc_gc(sliced_seqs)
        merged = ap.merge_dfs(gc_calc, self.primer_file, self.seqs)
        plate_order_sheet_f, plate_order_sheet_r = ap.to_order_plate(merged)
        self.assertEqual(len(plate_order_sheet_f), 13)
        self.assertEqual(len(plate_order_sheet_r), 13)

    def tearDown(self):
        pass

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

"""
Dependencies required to run program:
 - python3.6+
    - pandas>=0.22.0
    - argparse
    - sequence_info
A subset of functions used after PCR.
"""

import argparse
import os
import sys
import pandas as pd
from primer_tk import sequence_info as seqinf

def add_post_subparser(subparser):
    """ Add subparser for postprocessing step.

    Args:
        subparser (?): Subparser object.

    Returns: None
    """
    parser = subparser.add_parser("post", help='Parses info from pcr_output')
    parser.add_argument("-i", "--pcr_output",
                        dest="pcrfile", default="pcr_output.txt",
                        help="use output of isPCR")

    parser.add_argument("-tp", "--total_primers",
                        help="the pre-PCR master primer file that\
                              contains all sample + primer info.")
    parser.add_argument("-ot", "--off_target_max", default=8, type=int,
                        help="the maximum number of off target hits of primer.")
    parser.add_argument("-pcri", "--pcr_product_info", default="pcr_product_info.csv",
                        help="the information of all products generated by isPcr")
    parser.add_argument("-all", "--all_primer_info", default="all_final_primers.csv",
                        help="all of the successful primers generated")
    parser.add_argument("-top", "--top_final_primers", default="top_final_primers.csv",
                        help="the top primers generated for each position")
    parser.add_argument("-plate", "--plate_basename", default="plated_primers",
                        help="the basename of the primers in plate format ready to order.")


def fasta_parser(pcrfile):
    """
    Iterates through sudo-fasta output from in silico pcr
    and extracts all useful info related to samples.
    Args:
        pcrfile (file): file output from in silico pcr of primers.
    Returns:
        seqs (list): list of seqs from pcr output
        headers (list): list of headers from pcr output
    """
    seqs = []
    headers = []
    if not os.path.exists(pcrfile):
        sys.exit("ERROR: pcr_file '%s' DNE"%(pcrfile))
    with open(pcrfile) as pcr_file:
        sequence = ""
        header = None
        for line in pcr_file:
            if line.startswith('>'):
                headers.append(line[1:-1])
                if header:
                    seqs.append([sequence])
                sequence = ""
                header = line[1:]
            else:
                sequence += line.rstrip()
        seqs.append([sequence])
    return seqs, headers

def gc_percent_seqs(seqs_list):
    """
    Use seqinf to calculate GC content for output from isPCR.
    Args:
        seqs_list (list): list of seqs output from isPCR
    Returns:
        gc_list (list): list of gc content of seqs.
    """
    upper_seqs = [[nuc.upper() for nuc in seq] for seq in seqs_list]
    gc_list = []
    for seqs in upper_seqs:
        for seq in seqs:
            gc_list.append(seqinf.Sequence(seq).gc_percent())
    return gc_list

def split_headers_list(headers):
    """
    Split the headers list into unique elements instead of 1 string.
    Args:
        headers (list): headers output from fasta_parser
    Returns:
        split_headers (list): headers list split into components.
        no_chrom (list): all things that are not the chromosome.
    """
    split_headers = [item.split(" ") for item in headers]
    no_chrom = [item[1:] for item in split_headers]
    return split_headers, no_chrom

def chr_split_list(split_headers):
    """
    Gets chromosome info from split_headers.
    Args:
        split_headers (list): header list split into parts
    Returns:
        chr_list (list): list of chromosome values
    """
    chr_list = []
    for item in split_headers:
        if item[0]:
            new_item = item[0].split(":")
            chr_list.append(new_item)
    return chr_list

def pos_split_list(chr_list):
    """
    Gets position info from chr_list.
    Args:
        chr_list (list): list of chr info with position
    Returns:
        pos_list (list): list with position info
    """
    pos_list = []
    for pos in chr_list:
        if "+" in pos[1]:
            for_pos = pos[1].split("+")
            pos_list.append(for_pos)
        if "-" in pos[1]:
            rev_pos = pos[1].split("-")
            pos_list.append(rev_pos)
    return pos_list


def split_name_pos(no_chrom_list):
    """
    Splits the name and position in no_chrom_list.
    Args:
        no_chrom_list (list): list with header info not containing chromosome
    Returns:
        name_pos_split_list: split name and position list
    """
    name_pos_split_list = []
    for item in no_chrom_list:
        new_split = item[0].replace("_", "").split(":")
        name_pos_split_list.append([new_split[1]])
    return name_pos_split_list

def merge_info(chr_list, pos_split, name_pos_split_list, no_chrom_list):
    """
    Merges all lists created above to be used for DataFrame.
    Args:
        chr_list (list): the list with the chromosome info of product
        pos_split_list (list): the list with the split positions
        name_pos_split_list (list): the list with the names and positions split
        no_chrom_list (list): the list without chrom info
    Returns:
        merged_list (list): list with all info to be taken into dataframe
    """
    merged_list = []
    for chrom, pos, name, prims in zip(chr_list, pos_split, name_pos_split_list,
                                       no_chrom_list):
        merged_items = chrom + pos + name + prims
        merged_list.append(merged_items)
    return merged_list

def generate_pcr_df(merged_list, gc_list):
    """
    Generate a DataFrame from all the relevant merged_list information.
    Args:
        merged_list (list): list of all information about pcr products
    Returns:
        pcr_df (DataFrame): dataframe with all PCR product information
        good_primers_df (DataFrame): dataframe with primers that are on target
        bad_primers_df (DataFrame): dataframe with primers that failed
    """
    pcr_df = pd.DataFrame.from_records(merged_list)
    pcr_df.columns = ['Chromosome', 'Position1', 'Position2', 'Sample_Pos',
                      'Sequence ID', 'PCR_Prod_Len', 'Forward', 'Reverse']
    pcr_df['Product GC%'] = gc_list
    pcr_df['ForwardPrimLen'] = [len(primer) for primer in pcr_df['Forward']]
    pcr_df['ReversePrimLen'] = [len(primer) for primer in pcr_df['Reverse']]
    pcr_df['Position1'] = pcr_df['Position1'].astype(int)
    pcr_df['Position2'] = pcr_df['Position2'].astype(int)
    pcr_df['Sample_Pos'] = pcr_df['Sample_Pos'].astype(int)
    # Filter primers based on if they amplified in correct position
    off_target_dict = {}
    for fprim, pos1, pos2, sample_pos in zip(pcr_df['Forward'], pcr_df['Position1'],
                                             pcr_df['Position2'], pcr_df['Sample_Pos']):
        if fprim not in off_target_dict.keys() and (pos1 < sample_pos < pos2):
            off_target_dict[fprim] = 0
        elif fprim not in off_target_dict.keys() and not (pos1 < sample_pos < pos2):
            off_target_dict[fprim] = 1
        elif fprim in off_target_dict.keys() and not (pos1 < sample_pos < pos2):
            off_target_dict[fprim] += 1
    pcr_df['OffTargetCount'] = pcr_df['Forward'].map(off_target_dict)
    good_primers_df = pcr_df.loc[(pcr_df['Position1'] <= pcr_df['Sample_Pos']) &
                                 (pcr_df['Sample_Pos'] <= pcr_df['Position2'])]
    # These primers failed filtering
    bad_primers_df = pcr_df.loc[~((pcr_df['Position1'] <= pcr_df['Sample_Pos']) &
                                  (pcr_df['Sample_Pos'] <= pcr_df['Position2']))]

    return pcr_df, good_primers_df, bad_primers_df

def merge_good_total(good_primers, total_primers):
    """
    Merges good primers and total primers on Sequence ID.
    Picks highest ranking primers.
    Args:
        good_primers (file): dataframe generated by pcr analysis
        total_primers (file): dataframe generated by Primer3
    Returns:
        merged_df (DataFrame): good_total merged dataframe on Sequence ID
    """
    total_primers_df = pd.read_csv(total_primers)
    merged_df = total_primers_df.merge(good_primers, on='Sequence ID', how='left')
    merged_df['FwdPrimerPos'] = merged_df['Position1'].apply(str) + '-' +\
        (merged_df['Position1'] + merged_df['Primer Left Len']).apply(str)
    merged_df['RvsPrimerPos'] = (merged_df['Position2'] - merged_df['Primer Right Len']).apply(str)\
        + '-' + merged_df['Position2'].apply(str)
    return merged_df

def filter_merged(merged_df, off_target_max):
    """
    Filters the merged df to find only primers which align with initial primer pairs, and have a value
    below off target threshold.
    Args:
        merged_df (DataFrame): output df from merge_good_total()
        off_target_max (int): the maximum number of off target sites allowed (<= this value)
    Returns:
        off_target_filtered (DataFrame): the filtered dataframe
    """
    filtered_df = merged_df.loc[(merged_df['Primer Left Seq'] == merged_df['Forward']) &
                                (merged_df['Primer Right Seq'] == merged_df['Reverse'])]
    useful_filtered = filtered_df[['Sequence ID', 'Primer Rank', 'Primer Left Seq',
                                   'Primer Right Seq', 'Primer Left Len', 'Primer Right Len',
                                   'Primer Left TM', 'Primer Right TM', 'Primer Left GC %',
                                   'Primer Right GC %', 'Chromosome', 'Position1',
                                   'Position2', 'PCR_Prod_Len',
                                   'Product GC%', 'FwdPrimerPos', 'RvsPrimerPos', 'OffTargetCount']]
    off_target_filtered = useful_filtered.loc[useful_filtered['OffTargetCount'] <= off_target_max]
    return off_target_filtered

def top_ranked_final_primers(filter_merged_df):
    """
    Drops duplicate sequence ids and keeps first (which also corresponds)
    to the highest ranking primer pair for each sample.
    Args:
        filter_merged_df (DataFrame): input from filter_merged, where primers are only equal
                                      to on target primers from initial primer generation.
    Returns:
        top_ranked_df (DataFrame): outputs only the highest scoring primer pair
                                   at each position
    """
    top_ranked_df = filter_merged_df.drop_duplicates('Sequence ID', keep='first')
    return top_ranked_df

def to_order_plate(top_ranking_final_primers):
    """
    Takes output from top_ranked_final_primers and organizes to easy order IDT plates
    (Forward and Reverse).
    Args:
        top_ranked_final_primers (DataFrame): filtered, top ranked dataframe primers
    Returns:
        idt_order_sheet_plate_f (DataFrame): 96 well plate order format forward primers
        idt_order_sheet_plate_r (DataFrame): 96 well plate order format reverse primers
    """
    # Generate well numbers
    well_and_nums = []
    wells = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
    num_wells = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12']
    for well in wells:
        for num in num_wells:
            well_and_nums.append(well+num)

    # this is required because sometimes more than 96 primers are generated and the list is consumed.
    big_well_nums = well_and_nums * 50
    filt_ranked_df = top_ranking_final_primers
    plate_order_sheet_f = pd.DataFrame(columns=['Sequence Name', 'Sequence'])
    plate_order_sheet_r = pd.DataFrame(columns=['Sequence Name', 'Sequence'])
    plate_order_sheet_f['Sequence Name'] = filt_ranked_df['Sequence ID']+'_F'
    plate_order_sheet_f['Sequence'] = filt_ranked_df['Primer Left Seq']
    plate_order_sheet_f.insert(0, 'Well Position', big_well_nums[:len(plate_order_sheet_f['Sequence'])])
    plate_order_sheet_r['Sequence Name'] = filt_ranked_df['Sequence ID']+'_R'
    plate_order_sheet_r['Sequence'] = filt_ranked_df['Primer Right Seq']
    plate_order_sheet_r.insert(0, 'Well Position', big_well_nums[:len(plate_order_sheet_r['Sequence'])])
    return plate_order_sheet_f, plate_order_sheet_r



#!/usr/bin/env python3
"""
Modules required for program.
 - python3.6+
    - pandas>=0.22.0
    - argparse
    - sequence_info
"""

import argparse
from primer_tk import sequence_info as seqinf

def add_pre_subparser(subparser):
    """ Add subparser for 'pre' step.

    Args:
        subparser (?): Subparser object.

    Returns: None
    """
    parser = subparser.add_parser("pre", help="Preprocessing for snv/indel",
                                  description="Command Line argument for total primer"
                                              "input file to check if primers have a degree"
                                              "of complementarity with each other as defined"
                                              "by the user. Default is 60% (fairly strict).")

    parser.add_argument("-d", "--primer3_dump", dest="dump", required=True,
                        help="Primer3 stdout passed into a 'dump' file to be used as input")

    parser.add_argument("-o", "--outfile_name", dest="outfile", required=True,
                        help="The output filename for all primer information.")
    parser.add_argument("-nd", "--no_dimer", default="no_dimer_df.csv",
                        help="The primers left after dimers removed.")
    parser.add_argument("-spcr", "--standard_pcr_file", default="standard_pcr.txt",
                        help="The file to be used for standard pcr input")
    parser.add_argument("-mpcr", "--multiplex_pcr_file", default="multiplex_pcr.txt",
                        help="The file to be used for multiplex pcr input")
    parser.add_argument("-pa", "--percent_alignment", dest="percent_alignment",
                        default="60", help="Percent match between 2 primers for pair to be\
                                            discarded. EX: primer_len = 22, percent_aln = 60\
                                            dimer_len = (60/100) * 22 = 13.2 -> 13.")
    parser.add_argument("-pcr", "--pcr_type", dest="pcr", required=True,
                        choices=['standard', 'multiplex'],
                        help="perform standard or multiplex pcr on given inputs.")

def add_pre_sv_subparser(subparser):
    """ Add subparser for 'pre' step.

    Args:
        subparser (?): Subparser object.

    Returns: None
    """
    parser = subparser.add_parser("pre_sv", help="Preprocessing for SV's")

    parser.add_argument("-d", "--primer3_dump", dest="dump", required=True,
                        help="Primer3 stdout passed into a 'dump' file to be used as input")

    parser.add_argument("-o", "--outfile_name", dest="outfile", required=True,
                        help="The output filename for all primer information.")
    parser.add_argument("-pcr", "--pcrfile", default="standard_pcr.txt",
                        help="The pseudopcr file")

def get_fprimer_percent_aln(fprimer, percent_alignment):
    """
    Gets the len of the fprimer and calculates minimum percent alignment
    based on user input of maximum alignment between 2 primers allowed.
    See percent alignment help for info.
    Args:
        fprimer (str): the forward primer
    Returns:
        fp_len (int): user defined total number of bases to allowed to match.
    """
    fp_len = []
    for fseq in fprimer:
        f_len = len(fseq)
        min_dimer_alignment = int(f_len*(percent_alignment/100))
        fp_len.append(min_dimer_alignment)
    return fp_len

def primer_dimer_local(aln_len_list, names, seq1, seq2):
    """
    Looks for complementarity between all primers in list,
    and returns alignment + score. No return, prints visualization
    between aligned primers (forming dimer) if they meet the min score set in the func.
    Args:
        min_dimer_alignment (int): the minimum alignment score between 2 primers before
            it is thrown out
        names (string): sequence ID
        seq1 (string): primer1 sequence
        seq2 (string): primer2 sequence
    Returns:
        name, seq1, seq2, score (generator): yields any primer pair that has score
        greater than or equal to min_dimer_alignment.
    """
    for name, fseq, aln_len in zip(names, seq1, aln_len_list):
        for rseq in seq2:
            pd_test = seqinf.PrimerDimer(fseq, seqinf.Sequence(rseq).complement(), aln_len)
            pd_output = pd_test.pd_local()
            for item in pd_output:
                yield(name+"\n", seqinf.PrimerDimer.format_alignment_compl(item[0],
                                                                           item[1],
                                                                           item[2],
                                                                           item[3],
                                                                           item[4]))

def list_from_gen(gen_expression):
    """
    Creates a list from the generator function for dimer alignment.
    Items in each_item are name, seq1, seq2, score.
    Args:
        gen_expression (generator object): the output from primer_dimer_local
    Returns:
        primer_compare_list (list): generaor converted to list
    """
    primer_compare_list = []
    for each_item in gen_expression:
        primer_compare_list.append(str(each_item))
    return primer_compare_list

def p_list_formatter(primer_list):
    """
    Reformat the primer list (remove unnecessary characters from biopython2 output).
    Args:
        primer_list (list): list from list_from_gen output
    Returns:
        primer_dimers (list): list with unnecessary chars removed.
    """
    reformat_p_list = []
    primer_dimers = []
    reformat_p_list = [each_item.replace('\\n', ' ').split() for each_item in primer_list]
    for each_item in reformat_p_list:
        primer_dimers.append((each_item[0].replace('(', '').replace('\'', ''),
                              each_item[2].replace('\'', ''), each_item[4],
                              each_item[5].replace('\')', '')))
    return primer_dimers

def dimer_true(dataframe, col_num, dimer_list):
    """
    Boolean masks to let us know which primers from original df
    form dimers. If so they are dropped.
    Args:
        dataframe (pd.DataFrame): the primer dataframe
        col_num (int): the column number to check for primer match
        dimer_list (list): the list containing primer dimer info
    Returns:
        out_series (pd.Series): boolean masked series, True if primer is dimer, else False
    """
    out_series = dataframe.iloc[:, col_num].isin([seq[1] or seq[2] for seq in dimer_list])
    return out_series

def all_vs_all_pcr(df_boolean, outfile):
    """
    Creates all vs all pcr input to check for off target PCR amplification.
    This function assumes you used the output created from primer_cross_hyb.py
    Args:
        df_boolean (pd.DataFrame): dataframe with removed cross hybridizing primers
    Returns:
        nothing: writes a file output
    """
    no_dimer_df = df_boolean
    all_vs_all = open(outfile, 'w')
    for seqid, primer_left in zip(no_dimer_df['Sequence ID'], no_dimer_df['Primer Left Seq']):
        for primer_right in no_dimer_df['Primer Right Seq']:
            all_vs_all.write(str(seqid) + '\t' + str(primer_left) + '\t' + str(primer_right) + '\n')

    for seqid, primer_right in zip(no_dimer_df['Sequence ID'], no_dimer_df['Primer Right Seq']):
        for primer_left in no_dimer_df['Primer Left Seq']:
            all_vs_all.write(str(seqid) + '\t' + str(primer_right) + '\t' + str(primer_left) + '\n')

    for seqid, primer_left in zip(no_dimer_df['Sequence ID'], no_dimer_df['Primer Left Seq']):
        for primer_left2 in no_dimer_df['Primer Left Seq']:
            all_vs_all.write(str(seqid) + '\t' + str(primer_left) + '\t' + str(primer_left2) + '\n')

    for seqid, primer_right in zip(no_dimer_df['Sequence ID'], no_dimer_df['Primer Right Seq']):
        for primer_right2 in no_dimer_df['Primer Right Seq']:
            all_vs_all.write(str(seqid) + '\t' + str(primer_right) +\
                                 '\t' + str(primer_right2) + '\n')

    all_vs_all.close()

def standard_pcr(primer_df, outfile):
    """
    Creates standard PCR input file (one primer with one other) to run in silico PCR.
    Args:
        primer_df (pd.DataFrame): dataframe with primer info
    Returns:
        nothing: writes a file output
    """
    standard = open(outfile, 'w')
    for seqid, primer_left, primer_right in zip(primer_df['Sequence ID'],
                                                primer_df['Primer Left Seq'],
                                                primer_df['Primer Right Seq']):
        standard.write(str(seqid) + '\t' + str(primer_left) + '\t' + str(primer_right) + '\n')
    standard.close()



#!/usr/bin/env python
"""
Modules required to run program.
Dependencies:
 - python3.6+
    - pandas>=0.22.0
    - numpy>=1.14.2
    - biopython>=1.70
    - argparse
"""

import sys
import argparse

from Bio import SeqIO
import pandas as pd
from primer_tk import sequence_info as seqinf

def add_iterator_subparser(subparser):
    """ Get commandline arguments
    Args:
        subparser (?): Argparse subparsers

    Returns: None
    """

    parser = subparser.add_parser("iterator_sv", help="iterator_sv subparser")

    parser.add_argument("-ref", "--ref_genome", required=True,
                        help="Reference Genome File to design primers around")
    parser.add_argument("-in", "--regions_file", required=True,
                        help="File with regions to design primers around")
    parser.add_argument("-opt_size", "--primer_opt_size",
                        dest="primer_opt_size", default="22",
                        help="The optimum primer size for output, default: 22")
    parser.add_argument("-min_size", "--primer_min_size",
                        dest="primer_min_size", default="18",
                        help="The optimum primer size for output, default: 18")
    parser.add_argument("-max_size", "--primer_max_size",
                        dest="primer_max_size", default="25",
                        help="The optimum primer size for output, default: 25")
    parser.add_argument("-opt_gc", "--primer_opt_gc",
                        dest="primer_opt_gc", default="50",
                        help="Optimum primer GC, default: 50")
    parser.add_argument("-min_gc", "--primer_min_gc",
                        dest="primer_min_gc", default="20",
                        help="Minimum primer GC, default: 20")
    parser.add_argument("-max_gc", "--primer_max_gc",
                        dest="primer_max_gc", default="80",
                        help="Maximum primer GC, default: 80")
    parser.add_argument("-opt_tm", "--primer_opt_tm",
                        dest="primer_opt_tm", default="60",
                        help="Optimum primer TM, default: 60")
    parser.add_argument("-min_tm", "--primer_min_tm",
                        dest="primer_min_tm", default="57",
                        help="minimum primer TM, default: 57")
    parser.add_argument("-max_tm", "--primer_max_tm",
                        dest="primer_max_tm", default="63",
                        help="maximum primer TM, default: 63")
    parser.add_argument("-sr", "--product_size_range",
                        dest="product_size_range", default="200-400",
                        help="Size Range for PCR Product, default=200-400")
    parser.add_argument("-flank", "--flanking_region_size",
                        dest="flanking_region_size", default="200",
                        help="This value will select how many bases up and downstream to count\
                              when flanking SNP (will do 200 up and 200 down), default: 200")
    parser.add_argument("-st", "--sequence_target",
                        dest="sequence_target", default="199,1",
                        help="default: 199,1, should be half of your flanking region size,\
                              so SNP/V will be included.")
    parser.add_argument("-mp", "--mispriming", dest="mispriming", required=True,
                        help="full path to mispriming library for primer3\
                              (EX: /home/dkennetz/mispriming/humrep.ref")
    parser.add_argument("-tp", "--thermopath", dest="thermopath", required=True,
                        help="full path to thermo parameters for primer3 to use\
                              (EX: /home/dkennetz/primer3/src/primer3_config/) install loc")
    parser.add_argument("-sv", "--sv-type", dest="sv",
                        choices=['deletion', 'inversion', 'insertion'], required=True,
                        help="currently supported SV primer generation: "
                             "deletion, inversion, and insertion.")

def genome_iterator(genome):
    """
    Uses Biopython SeqIO to parse a genome Fasta and store each chr and
    sequence as a list.

    Args:
        genome (file): Full reference genome from command-line arguments.

    Returns:
        output (list): genome parsed into list of tuples of (header, seq)
        by chromosome for easy access later.
    """

    output = []
    with open(genome, "r") as handle:
        for record in SeqIO.parse(handle, "fasta"):
            chrm = record.id
            seq = record.seq
            output.append((chrm, seq))
        return output

def create_dataframe_csv(regions_file):
    """
    Creates a pandas DataFrame from a regions file in comma separated form
    and names the columns in the DF according to their values.

    Args:
        regions_file (file): Full path to -in input regions.csv.
        This should have coordinate positions of interest to design primers around.
        Note: the chromosome column can be of format chr1 or simply 1 (chr not necessary).
        The file should contain no headers and should be structured as follows:

        Gene1,Sample1,chr1,pos1,pos2
        Gene2,Sample2,chr2,pos1,pos2
        Gene3,Sample3,chr3,pos1,pos2
        ...
    Returns:
        regions_df (pd.DataFrame): The infile parsed to pd.DataFrame object.
    """

    regions_df = pd.read_csv(regions_file, header=None)
    regions_df.columns = ['Gene', 'Sample', 'Chr', 'PosStart', 'PosStop']
    regions_df = regions_df.astype({'Chr': str})
    return regions_df

def create_dataframe_insertion_csv(regions_file):
    """
    Creates a pandas DataFrame from a regions file in comma separated form
    and names the columns in the DF according to their values.

    Args:
        regions_file (file): Full path to -in input regions.csv.
        This should have coordinate positions of interest to design primers around.
        Note: the chromosome column can be of format chr1 or simply 1 (chr not necessary).
        The file should contain no headers and should be structured as follows:

        Gene1,Sample1,chrNorm1,posNorm1,posNorm2,chrIns2,posIns1,posIns2,strand
        Gene2,Sample2,chrNorm2,posNorm1,posNorm2,chrIns3,posIns1,posIns2,strand
        Gene3,Sample3,chrNorm3,posNorm1,posNorm2,chrIns4,posIns1,posIns2,strand
        ...
    Returns:
        regions_df (pd.DataFrame): The infile parsed to pd.DataFrame object.
    """

    regions_df = pd.read_csv(regions_file, header=None)
    regions_df.columns = ['Gene', 'Sample', 'ChrNorm', 'PosNorm1', 'PosNorm2',
                          'ChrIns', 'PosIns1', 'PosIns2', 'Strand']
    regions_df = regions_df.astype({'ChrNorm': str})
    regions_df = regions_df.astype({'ChrIns': str})
    return regions_df

def create_dataframe_txt(regions_file):
    """
    Creates a pandas DataFrame from a regions file in a tab separated form
    and uses the following column names in the DF according to their values.

    Args:
        regions_file (file): Full path to -in input regions.txt.
        This should have coordinate positions of interest to design primers around.
        Note: the chromosome column can be of format chr1 or simply 1 (chr not necessary).
        The file should contain no headers and should be constructed as follows:

        Gene1\tSample1\tchr1\tpos1\tpos2
        Gene2\tSample2\tchr2\tpos1\tpos2
        Gene3\tSample3\tchr3\tpos1\tpos2
        ...
    Returns:
        regions_df (pd.DataFrame): The infile parsed to pd.DataFrame object.
    """
    regions_df = pd.read_table(regions_file, header=None)
    regions_df.columns = ['Gene', 'Sample', 'Chr', 'PosStart', 'PosStop']
    regions_df = regions_df.astype({'Chr': str})
    return regions_df

def create_dataframe_insertion_txt(regions_file):
    """
    Creates a pandas DataFrame from a regions file in a tab separated form
    and uses the following column names in the DF according to their values.

    Args:
        regions_file (file): Full path to -in input regions.txt.
        This should have coordinate positions of interest to design primers around.
        Note: the chromosome column can be of format chr1 or simply 1 (chr not necessary).
        The file should contain no headers and should be constructed as follows:

        Gene1\tSample1\tchrNorm1\tposNorm1\tposNorm2\tchrIns2\tposIns1\tposIns2\tstrand
        Gene2\tSample2\tchrNorm2\tposNorm1\tposNorm2\tchrIns3\tposIns1\tposIns2\tstrand
        Gene3\tSample3\tchrNorm3\tposNorm1\tposNorm2\tchrIns4\tposIns1\tposIns2\tstrand
        ...
    Returns:
        regions_df (pd.DataFrame): The infile parsed to pd.DataFrame object.
    """
    regions_df = pd.read_table(regions_file, header=None)
    regions_df.columns = ['Gene', 'Sample', 'ChrNorm', 'PosNorm1', 'PosNorm2',
                          'ChrIns', 'PosIns1', 'PosIns2', 'Strand']
    regions_df = regions_df.astype({'ChrNorm': str})
    regions_df = regions_df.astype({'ChrIns': str})
    return regions_df

def file_extension(infile, strvar):
    """
    Runs pd.read_ function to create dataframe corresponding
    to file extension, .txt and .csv accepted.
    This will call the function to create a df only if file extension is correct.
    If the file extension is wrong, it will return the error message and exit.

    Args:
        infile (file): Full path to -in input regions.txt.
        strvar (string): the structural variant type
    Returns:
        small_regions (pd.DataFrame): The infile parsed to pd.DataFrame object
    """
    if infile.lower().endswith('.txt') and strvar in ('deletion', 'inversion'):
        small_regions = create_dataframe_txt(infile)
    elif infile.lower().endswith('.csv') and strvar in ('deletion', 'inversion'):
        small_regions = create_dataframe_csv(infile)
    elif infile.lower().endswith('.txt') and strvar == 'insertion':
        small_regions = create_dataframe_insertion_txt(infile)
    elif infile.lower().endswith('.csv') and strvar == 'insertion':
        small_regions = create_dataframe_insertion_csv(infile)
    else:
        sys.exit("Wrong File Format, should be .txt (tab) or .csv (comma), or check sv type.")
    return small_regions

def match_chr_to_genome(dataframe, genome, strvar):
    """
    Used to match formatting between regionsFile input and genome for chromosome
    denotations. Some genomes contain the string "chr" and some do not.

    Args:
        dataframe (pandas object): dataframe created from create_dataframe() function.
        genome (list): genome list of tuples created from genome_iterator() function.

    Returns:
        dataframe (pandas object): dataframe formatted to match genome annotation of chromosome.
    """
    if strvar in ('deletion', 'inversion'):
        if dataframe['Chr'].str.contains("chr").any() and "chr" not in str(genome[0][0]):
            dataframe['Chr'] = dataframe['Chr'].str.replace('chr', '')
        elif not dataframe['Chr'].str.contains("chr").any() and "chr" in str(genome[0][0]):
            dataframe['Chr'] = 'chr' + dataframe['Chr']
    elif strvar == 'insertion':
        if dataframe['ChrNorm'].str.contains("chr").any() and "chr" not in str(genome[0][0]):
            dataframe['ChrNorm'] = dataframe['ChrNorm'].str.replace('chr', '')
            dataframe['ChrIns'] = dataframe['ChrIns'].str.replace('chr', '')
        elif not dataframe['ChrNorm'].str.contains("chr").any() and "chr" in str(genome[0][0]):
            dataframe['ChrNorm'] = 'chr' + dataframe['ChrNorm']
            dataframe['ChrIns'] = 'chr' + dataframe['ChrIns']
    else:
        sys.exit("Wrong SV type, please select an SV in help menu.")
    return dataframe

def flanking_regions_fasta_deletion(genome, dataframe, flanking_region_size):
    """
    Makes batch processing possible, pulls down small region
    of genome for which to design primers around.
    This is based on the chromosome and position of input file.
    Each Fasta record  will contain:

    >Sample_Gene_chr:posStart-posStop
    Seq of flanking region upstream of SV + seq of flanking region downstream of SV

    Args:
        genome (list): genome list of tuples (header, seq).
        dataframe (pandas object): dataframe with sample info.
        flanking_region_size (int): length of sequence upstream and downstream of
        input coordinate position to pull as sequence to design primers around.
    """
    output = []
    for headers, seqs in genome:
        chrm = str(headers)
        seq = str(seqs)
        for gene, sample, chrom, start, stop in zip(dataframe.Gene, dataframe.Sample, dataframe.Chr,
                                                    dataframe.PosStart, dataframe.PosStop):
            if str(chrom) == chrm:
                header = str(str(sample)+"_"+str(gene)+"_"+\
                                 str(chrom)+":"+str(start)+"-"+str(stop)+"__")
                flank_seq = seq[int(start)-int(flanking_region_size):int(start)+1]\
                    +seq[int(stop):(int(stop)+int(flanking_region_size))]
                output.append((header, flank_seq.upper()))
    return output

def flanking_regions_fasta_inversion(genome, dataframe, flanking_region_size):
    """
    Makes batch processing possible, pulls down small region
    of genome for which to design primers around and generates
    flanking regions based on an inverted sequence.

    This is based on the chromosome and position of input file.
    Each Fasta record  will contain:

    >Sample_Gene_chr:posStart-posStop_BP
    Seq of flanking region upstream of SV + seq of flanking region downstream of SV

    Args:
        genome (list): genome list of tuples (header, seq).
        dataframe (pandas object): dataframe with sample info.
        flanking_region_size (int): length of sequence upstream and downstream of
        input coordinate position to pull as sequence to design primers around.
    Returns:
        output (list): header + seq
    """
    output = []
    for headers, seqs in genome:
        chrm = str(headers)
        seq = str(seqs)
        for gene, sample, chrom, start, stop in zip(dataframe.Gene, dataframe.Sample, dataframe.Chr,
                                                    dataframe.PosStart, dataframe.PosStop):
            if str(chrom) == chrm:
                header = str(str(sample)+"_"+str(gene)+"_"+\
                                 str(chrom)+":"+str(start)+"-"+str(stop)+"__BP1")
                flank_seq = seq[int(start)-int(flanking_region_size):int(start)+1]\
                    + seqinf.\
                    Sequence(seq[int(stop):((int(stop)-(int(flanking_region_size)+1))):-1])\
                    .complement()
                output.append((header, flank_seq))
        for gene, sample, chrom, start, stop in zip(dataframe.Gene, dataframe.Sample, dataframe.Chr,
                                                    dataframe.PosStart, dataframe.PosStop):
            if str(chrom) == chrm:
                header = str(str(sample)+"_"+str(gene)+"_"+\
                                 str(chrom)+":"+str(start)+"-"+str(stop)+"__BP2")
                flank_seq = seqinf.\
                    Sequence(seq[int(start)+int(flanking_region_size):int(start)+1:-1])\
                    .complement()\
                    + seq[int(stop):(int(stop)+int(flanking_region_size))]
                output.append((header, flank_seq.upper()))
    return output

def flanking_region_fasta_insertion(genome, dataframe, flanking_region_size):
    """
    Makes batch processing possible, pulls down small region
    of genome for which to design primers around and generates
    flanking regions based on an inverted sequence.

    This is based on the chromosome and position of input file.
    Each Fasta record  will contain:

    >Sample_Gene_chr:posNorm1-posNorm2_BP
    Seq of flanking region upstream of SV + seq of inserted sequence based on strand

    Args:
        genome (list): genome list of tuples (header, seq)
        dataframe (pandas object): dataframe with sample info.
        flanking_region_size (int): length of sequence upstream dna downstream of
        input coordinate position to pull as sequence to design primers around.
    Returns:
        output (list): (header + seq)
    """
    headersbp1 = []
    seqsnormbp1 = []
    seqsinsbp1 = []
    headersbp2 = []
    seqsnormbp2 = []
    seqsinsbp2 = []
    for headers, seqs in genome:
        chrm = str(headers)
        seq = str(seqs)
        for gene, sample, chrn, startn, stopn in zip(dataframe.Gene, dataframe.Sample,
                                                     dataframe.ChrNorm, dataframe.PosNorm1,
                                                     dataframe.PosNorm2):
            if str(chrn) == chrm:
                header = str(str(sample)+"_"+str(gene)+"_"+str(chrn)+":"+\
                                 str(startn)+"-"+str(stopn)+"__BP1")
                flank_seq = seq[int(startn)-int(flanking_region_size):int(startn)]
                seqsnormbp1.append(flank_seq)
                headersbp1.append(header)

        for chri, starti, stopi, strand in zip(dataframe.ChrIns, dataframe.PosIns1,
                                               dataframe.PosIns2, dataframe.Strand):
            if str(chri) == chrm and strand == '+':
                flank_seq = seq[int(starti):int(starti)+int(flanking_region_size)]
                seqsinsbp1.append(flank_seq)
            elif str(chri) == chrm and strand == '-':
                flank_seq = seqinf.\
                    Sequence(seq[int(stopi):int(stopi)-(int(flanking_region_size)):-1])\
                    .complement()
                seqsinsbp1.append(flank_seq)

        for gene, sample, chrn, startn, stopn in zip(dataframe.Gene, dataframe.Sample,
                                                     dataframe.ChrNorm,
                                                     dataframe.PosNorm1,
                                                     dataframe.PosNorm2):
            if str(chrn) == chrm:
                header = str(str(sample)+"_"+str(gene)+"_"+str(chrn)+":"+\
                                 str(startn)+"-"+str(stopn)+"__BP2")
                flank_seq = seq[int(stopn):int(stopn)+int(flanking_region_size)]
                seqsnormbp2.append(flank_seq)
                headersbp2.append(header)

        for chri, starti, stopi, strand in zip(dataframe.ChrIns, dataframe.PosIns1,
                                               dataframe.PosIns2, dataframe.Strand):
            if str(chri) == chrm and strand == '+':
                flank_seq = seq[int(stopi):int(stopi)-(int(flanking_region_size)):-1]
                seqsinsbp2.append(flank_seq)
            elif str(chri) == chrm and strand == '-':
                flank_seq = seqinf.\
                    Sequence(seq[int(starti):int(starti)+(int(flanking_region_size))])\
                    .complement()
                seqsinsbp2.append(flank_seq)

    output = []
    for headers, seqsnorm, seqsins in zip(headersbp1, seqsnormbp1, seqsinsbp1):
        combined_seq = seqsnorm + seqsins
        output.append((headers, combined_seq.upper()))

    for headers, seqsnorm, seqsins in zip(headersbp2, seqsnormbp2, seqsinsbp2):
        combined_seq = seqsins + seqsnorm
        output.append((headers, combined_seq.upper()))
    return output



#!/usr/bin/env python3

""" Modules required to run program.

Dependencies:
 - python3.6+
    - biopython>=1.70
"""

from Bio import pairwise2

class Sequence:
    """
    Creates a sequence object from an input DNA sequence.
    """

    def __init__(self, sequence):
        """
        Initialize seq object.
        """
        self.sequence = str(sequence).upper()
        self.name = "seq_name_not_set"

    def set_name(self, name):
        """
        Sets name of sequence object.
        """
        self.name = name

    def complement(self):
        """
        Returns complement (defined in Global vars).
        Example: ATCG -> TAGC
        """
        return ''.join(map(complement_base, self.sequence))

    def reverse_complement(self):
        """
        Returns the reversed complement of the sequence.
        Example: ATCG -> TAGC -> CGAT
        """
        return ''.join(map(complement_base, self.sequence[::-1]))

    def reverse_sequence(self):
        """
        Returns the reverse of the input sequence.
        Example: ATCG -> GCTA
        """
        return self.sequence[::-1]

    def gc_percent(self):
        """
        Calculates GC as a percentage of the input sequence.
        Example: ATCG = 50
        """
        return float((self.sequence.count('G') +\
                          self.sequence.count('C')) / float(len(self.sequence))*100)


class PrimerDimer(Sequence):
    """
    Identifies dimerization between 2 sequences. Essentially,
    it identifies high degrees of complementarity between
    sequences which could lead to undesired binding behavior
    during PCR amplification. Primers bind to each other rather
    than target DNA strand if in pool.
    Example: ATCG + TAGC (high degree of complementarity).
    """
    def __init__(self, sequence, compare_sequence, pd_length):
        super(PrimerDimer, self).__init__(sequence)
        self.compare_sequence = compare_sequence
        self.pd_length = pd_length

    def pd_local(self):
        """
        Compares sequence and comparesequence for the highest number of
        identical characters within a single alignment of the two strings.
        Output is a list with:
        [sequence,
        complement(comparesequence),
        score,
        beginning-of-alignment,
        end-of-alignment]
        Use of the static method 'format_alignment_compl' to format the
        output is highly recommended to display the alignments.
        """
        for alignment in pairwise2.align.localxs(self.sequence,
                                                 self.compare_sequence,
                                                 -len(self.sequence),
                                                 -len(self.sequence)):
            if int(alignment[2]) >= self.pd_length:
                yield alignment

    @staticmethod
    def format_alignment_compl(align1, align2, score, begin, end):
        """
        A modified version of biopython.pairwise2.format_alignment.
        Instead of returning the align2 argument, the complement of align2
        is returned.
        For PrimerDimers we are trying to find complementary base pairings
        so this scores based on complementarity, whereas pairwise2 scores on
        literal matches.
        """
        seq = []
        compalign2 = Sequence(align2).complement()

        seq.append("%s\n" % align1)
        seq.append("%s%s\n" % (" "*begin, "|"*(end-begin)))
        seq.append("%s\n" % compalign2)
        seq.append("  Score=%g\n" % score)
        return ''.join(seq)

# GLOBAL VARS
_COMPLEMENTS = {'-': '-', 'N': 'N', 'A': 'T', 'T': 'A', 'G': 'C', 'C': 'G'}

def is_complement(base1, base2):
    """
    Used to identify is base2 is a complement of base1
    """
    return _COMPLEMENTS[base1] == base2

def complement_base(base):
    """
    Returns the complement for a base.
    """
    if base in _COMPLEMENTS:
        return _COMPLEMENTS[base]
    print("base not in complements, something is wrong!")



#!/usr/bin/env python3

"""
Dependencies required to run program:
 - python3.6+
    - pandas>=0.22.0
    - numpy>=1.16.0
"""

import pandas as pd
import numpy as np
import sys

class MissingPrimers:
    """
    Groups Primer3 output by sample id into a list of lists,
    adds NA to all groups that do not have 5 primers generated,
    and then outputs a list corresponding to each primer rank.
    Args:
        file_name (string): name of file to be passed to program
        value (int): rank of primer pair to store as list [0-4] only
    Returns:
        samp_primer_info (list): list of lists with all values required for
                                 downstream analysis (id, lseq, rseq, l_tm, r_tm,
                                 l_gc, r_gc, product_size)
    """
    def __init__(self, file_name, value):
        """
        Initialize Values
        """
        self.file_name = file_name
        self.value = str(value)
        self.dump_list = self.__group_seqids()
        self.filled_primers = self.__fill_empty_values()
        self.samp_primer_info = self.__gather_primer_info()

    def __group_seqids(self):
        """
        Group by sample_id
        """
        sequence = ""
        with open(self.file_name) as dump:
            for line in dump:
                if not line.startswith('='):
                    sequence += line
                if line.startswith('='):
                    fixed_line = line.replace('=', '@@@')
                    sequence += fixed_line
                full_string = ''.join([line for line in sequence])
                primer_split = [[string] for string in full_string.\
                                    split('@@@') if string is not ' ']
        primers_info = primer_split[:-1]
        return primers_info

    def __fill_empty_values(self):
        """
        Fill in missing required values with NA
        """
        sequence = []
        for sample in self.dump_list:
            for string in sample:
                if "PRIMER_LEFT_0_SEQUENCE=" not in string:
                    string = string + "PRIMER_LEFT_0_SEQUENCE=NA\n"\
                        + "PRIMER_RIGHT_0_SEQUENCE=NA\n"\
                        + "PRIMER_LEFT_0=NA\n"\
                        + "PRIMER_RIGHT_0=NA\n"\
                        + "PRIMER_LEFT_0_TM=NA\n" + "PRIMER_RIGHT_0_TM=NA\n"\
                        + "PRIMER_LEFT_0_GC_PERCENT=NA\n"\
                        + "PRIMER_RIGHT_0_GC_PERCENT=NA\n"\
                        + "PRIMER_PAIR_0_PRODUCT_SIZE=NA\n"
                if "PRIMER_LEFT_1_SEQUENCE=" not in string:
                    string = string + "PRIMER_LEFT_1_SEQUENCE=NA\n"\
                        + "PRIMER_RIGHT_1_SEQUENCE=NA\n"\
                        + "PRIMER_LEFT_1=NA\n"\
                        + "PRIMER_RIGHT_1=NA\n"\
                        + "PRIMER_LEFT_1_TM=NA\n" + "PRIMER_RIGHT_1_TM=NA\n"\
                        + "PRIMER_LEFT_1_GC_PERCENT=NA\n"\
                        + "PRIMER_RIGHT_1_GC_PERCENT=NA\n"\
                        + "PRIMER_PAIR_1_PRODUCT_SIZE=NA\n"
                if "PRIMER_LEFT_2_SEQUENCE=" not in string:
                    string = string + "PRIMER_LEFT_2_SEQUENCE=NA\n"\
                        + "PRIMER_RIGHT_2_SEQUENCE=NA\n"\
                        + "PRIMER_LEFT_2=NA\n"\
                        + "PRIMER_RIGHT_2=NA\n"\
                        + "PRIMER_LEFT_2_TM=NA\n" + "PRIMER_RIGHT_2_TM=NA\n"\
                        + "PRIMER_LEFT_2_GC_PERCENT=NA\n"\
                        + "PRIMER_RIGHT_2_GC_PERCENT=NA\n"\
                        + "PRIMER_PAIR_2_PRODUCT_SIZE=NA\n"
                if "PRIMER_LEFT_3_SEQUENCE=" not in string:
                    string = string + "PRIMER_LEFT_3_SEQUENCE=NA\n"\
                        + "PRIMER_RIGHT_3_SEQUENCE=NA\n"\
                        + "PRIMER_LEFT_3=NA\n"\
                        + "PRIMER_RIGHT_3=NA\n"\
                        + "PRIMER_LEFT_3_TM=NA\n" + "PRIMER_RIGHT_3_TM=NA\n"\
                        + "PRIMER_LEFT_3_GC_PERCENT=NA\n"\
                        + "PRIMER_RIGHT_3_GC_PERCENT=NA\n"\
                        + "PRIMER_PAIR_3_PRODUCT_SIZE=NA\n"
                if "PRIMER_LEFT_4_SEQUENCE=" not in string:
                    string = string + "PRIMER_LEFT_4_SEQUENCE=NA\n"\
                        + "PRIMER_RIGHT_4_SEQUENCE=NA\n"\
                        + "PRIMER_LEFT_4=NA\n"\
                        + "PRIMER_RIGHT_4=NA\n"\
                        + "PRIMER_LEFT_4_TM=NA\n" + "PRIMER_RIGHT_4_TM=NA\n"\
                        + "PRIMER_LEFT_4_GC_PERCENT=NA\n"\
                        + "PRIMER_RIGHT_4_GC_PERCENT=NA\n"\
                        + "PRIMER_PAIR_4_PRODUCT_SIZE=NA\n"
                sequence.append(string.lstrip('\n').split('\n'))

        return sequence

    def __gather_primer_info(self):
        """
        Return the final list of lists with all filled values by rank
        """
        sample_info = []
        for item in self.filled_primers:
            for p_info in item:
                if "SEQUENCE_ID=" in p_info:
                    seq_id = p_info[12:]+self.value
                if "PRIMER_LEFT_%s_SEQUENCE=" %self.value in p_info:
                    p_left = p_info[23:]
                if "PRIMER_RIGHT_%s_SEQUENCE=" %self.value in p_info:
                    p_right = p_info[24:]
                if "PRIMER_LEFT_%s=" %self.value in p_info:
                    left_pos = p_info[14:-3]
                if "PRIMER_RIGHT_%s=" %self.value in p_info:
                    right_pos = p_info[15:-3]
                if "PRIMER_LEFT_%s_TM=" %self.value in p_info:
                    left_tm = p_info[17:]
                if "PRIMER_RIGHT_%s_TM=" %self.value in p_info:
                    right_tm = p_info[18:]
                if "PRIMER_LEFT_%s_GC_PERCENT=" %self.value in p_info:
                    left_gc = p_info[25:]
                if "PRIMER_RIGHT_%s_GC_PERCENT=" %self.value in p_info:
                    right_gc = p_info[26:]
                if "PRIMER_PAIR_%s_PRODUCT_SIZE=" %self.value in p_info:
                    product_size = p_info[27:]
                    sample_info.append([seq_id, p_left, p_right, left_tm,
                                        right_tm, left_gc, right_gc,
                                        product_size, left_pos,
                                        right_pos])

        return sample_info

def create_df(primer_lists):
    """
    Creates a dataframe from gather_primer_info
    Args:
        primer_lists (list): primer information lists to be passed for dataframe.
    Returns:
        primer_df (pd.DataFrame): Dataframe with all primer information.
    """
    primer_df = pd.DataFrame(np.ma.row_stack(primer_lists),
                             columns=['Sequence ID', 'Primer Left Seq', 'Primer Right Seq',
                                      'Primer Left TM', 'Primer Right TM',
                                      'Primer Left GC %', 'Primer Right GC %',
                                      'Primer3 Predicted Product', 'Primer Left Pos',
                                      'Primer Right Pos'])
    primer_df = primer_df.sort_values('Sequence ID').reset_index().drop(labels='index', axis=1)
    rank = [num[-1] for num in primer_df['Sequence ID']]
    primer_df.insert(1, 'Primer Rank', rank)
    left_len = [len(length) for length in primer_df['Primer Left Seq']]
    right_len = [len(length) for length in primer_df['Primer Right Seq']]
    primer_df.insert(4, 'Primer Left Len', left_len)
    primer_df.insert(5, 'Primer Right Len', right_len)
    primer_df['Sequence ID'] = [seqid[:-1] for seqid in primer_df['Sequence ID']]
    return primer_df



#!/usr/bin/env python3

"""
Dependencies required to run program.
 - python3.6+
    - argparse
    - pandas >= 0.22.4
    - pysam==0.15.2
"""

import sys
import pandas as pd
from primer_tk import constants

def add_tabix_subparser(subparser):
    """
    Get commandline arguments
    Args:
        subparser (?): Subparser object.
    Returns:
        args (Namespace): the parsed arguments.
    """
    parser = subparser.add_parser("tabix", help="Tabix subparser")
    parser.add_argument("-vcf", "--variant-call-file", dest="vcf",
                        help="Tabix indexed VCF.")
    parser.add_argument("-in", "--primer-input-file", dest="p_info",
                        help="The output of the primer pipeline.")
    parser.add_argument("-o", "--output", dest="output",
                        help="The name of the output file")
    return parser

def create_tabix_df(primer_pipeline_output):
    """
    Takes output of primer pipeline and generates dataframe.

    Args:
    total_primers (file): the output of the primer pipeline
    Returns:
    dataframe (pd.DataFrame): a pandas dataframe
    """
    primer_df = pd.read_csv(primer_pipeline_output, header=0)
    return primer_df

def primer_range_left(seqid, rank, chrm, p_left, position1):
    """
    Takes the chromosome, primer sequence, and position and creates a
    query range for tabix to search by.

    Args:
        seqid (pd.Series): pandas column containing seqids
        rank (pd.Series): pandas column containing primer rank
        chrm (pd.Series): pandas column of chrm info
        p_left (pd.Series): pandas column of the left primer seq
        position1 (pd.Series): pandas column of the left primer position.
    Returns:
        p_left_info (pd.DataFrame): the positional info in a frame format.
    """
    p_left_info = pd.DataFrame()
    p_left_info['Sequence ID'] = seqid
    p_left_info['Primer Rank'] = rank
    p_left_info['Chromosome'] = chrm.apply(str)
    p_left_info['P_Len'] = p_left.apply(len)
    p_left_info['Position1'] = position1
    p_left_info['Position2'] = p_left_info['Position1']\
        + p_left_info['P_Len']
    return p_left_info

def primer_range_right(seqid, rank, chrm, p_right, position2):
    """
    Takes the chromosome, primer sequence, and position and creates a
    query range for tabix to search by.

    Args:
        seqid (pd.Series): pandas column containing seqids
        rank (pd.Series): pandas column containing primer rank
        chrm (pd.Series): pandas column of chrm info
        p_right (pd.Series): pandas column of the right primer seq
        position1 (pd.Series): pandas column of the right primer position.
    Returns:
        p_right_info (pd.DataFrame): the positional info in a frame format.
    """
    p_right_info = pd.DataFrame()
    p_right_info['Sequence ID'] = seqid
    p_right_info['Primer Rank'] = rank
    p_right_info['Chromosome'] = chrm.apply(str)
    p_right_info['P_Len'] = p_right.apply(len)
    p_right_info['Position2'] = position2
    p_right_info['Position1'] = p_right_info['Position2']\
        - p_right_info['P_Len']
    return p_right_info


def match_pinfo_to_vcf(p_info, vcf):
    """
    Normalizes chromosome column to reference VCF info.
    Args:
        p_info (pd.DataFrame): the primer information df
        vcf (file): the tabix indexed vcf input
    Returns:
        p_info (pd.DataFrame): the primer information df normalized to genome
    """
    switch = 0
    try:
        for rec in vcf.fetch('chr1', constants.START_POS, constants.END_POS):
            print("Updating switch: 1")
        switch = 1
    except:
        for rec in vcf.fetch('1', constants.START_POS, constants.END_POS):
            print("Updating switch: 2")
        switch = 2

    if switch == 1 and not p_info['Chromosome']\
            .str.contains("chr").any():
        p_info['Chromosome'] = 'chr' + p_info['Chromosome']
    elif switch == 2 and p_info['Chromosome']\
            .str.contains("chr").any():
        p_info['Chromosome'] = p_info['Chromosome']\
            .str.replace('chr', '')
    else:
        pass
    return p_info


def tabix_fetch(seqids, ranks, chrom, position1, position2, vcf_in):
    """
    Takes p_info positions and fetches SNPs from tabix indexed VCF.
    Args:
        seqids (pd.Series): pandas column of seqids
        ranks (pd.Series): pandas column of primer ranks
        chrom (pd.Series): pandas column of chrm info
        position1 (pd.Series): pandas column of pos1 info
        position2 (pd.Series): pandas column of pos2 info
    Returns:
        snp_list (list): list containing vcf info for primer positions.
    """
    snp_list = []
    for seqid, rank, chrm, pos1, pos2 in zip(seqids, ranks, chrom, position1, position2):
        for row in vcf_in.fetch(chrm, pos1, pos2):
            snp_list.append((seqid, rank, str(row).strip('\n').split('\t')))
    select_list = []
    for item in snp_list:
        select_list.append((item[0], item[1], item[2][0], item[2][1],\
                                item[2][2], item[2][7].split(';')))
    final_list = []
    for item in select_list:
        geneinfo = [j for j in item[5] if "GENEINFO=" in j]
        caf = [j for j in item[5] if "CAF=" in j]
        topmed = [j for j in item[5] if "TOPMED=" in j]
        final_list.append((item[0], item[1], item[2], item[3], item[4], geneinfo, caf, topmed))
    return final_list

def tabix_results_to_df(tabix_list, which_primer, column_name):
    """
    Takes the results from the tabix search and creates a dataframe.
    Args:
        tabix_list (list): tabix info from primer_left
        which_primer (str): should be L or R to denote left or right in names
        column_name (str): column name of snp count (should specify left or right)
    Returns:
        tabix_frame (pd.DataFrame): list organized into dataframe
    """
    tabix_frame = pd.DataFrame(tabix_list)
    if len(tabix_frame) == 0:
        sys.exit("There are no SNPs in any of your primers, WOW!")
    else:
        pass
    tabix_frame.columns = ["Sequence ID", "Primer Rank",
                           "Chromosome", "SNPPosition", "rs_id",
                           "GeneInfo", "CommonAlleleFreq", "TopMedFreq"]
    tabix_frame["GeneInfo"] = tabix_frame["GeneInfo"]\
        .apply(lambda x: "NA" if len(x) == 0 else x[0].split("=")[1])
    tabix_frame["CommonAlleleFreq"] = tabix_frame["CommonAlleleFreq"]\
        .apply(lambda x: "NA" if len(x) == 0 else x[0].split('=')[1].split(',')[1])
    tabix_frame["TopMedFreq"] = tabix_frame["TopMedFreq"]\
        .apply(lambda x: "NA" if len(x) == 0 else x[0].split('=')[1].split(',')[1])
    tabix_frame = tabix_frame.groupby(["Sequence ID", "Primer Rank", "Chromosome"])\
        .agg({'SNPPosition': ';'.join,
              'rs_id': ';'.join,
              'GeneInfo': 'first',
              'CommonAlleleFreq': ';'.join,
              'TopMedFreq': ';'.join}).reset_index()
    tabix_frame[column_name] = tabix_frame["rs_id"]\
        .apply(lambda x: len(x.split(';')))
    tabix_frame.columns = ["Sequence ID", "Primer Rank", "Chromosome",
                           "%s_SNPPosition" %which_primer,
                           "%s_rs_id" %which_primer,
                           "%s_GeneInfo" %which_primer,
                           "%s_CommonAlleleFreq" %which_primer,
                           "%s_TopMedFreq" %which_primer, column_name]
    return tabix_frame

def merge_left_right(left_df, right_df, total):
    """
    Merge left and right primer tabix info dataframes.
    Args:
        left_df (pd.DataFrame): primer left tabix dataframe
        right_df (pd.DataFrame): primer right tabix dataframe
    returns:
        merged_tabix_df (pd.DataFrame): left and right merged df
    """
    merged_tabix_df = pd.merge(total, left_df,
                               on=['Sequence ID', 'Primer Rank'], how='left')\
                               .merge(right_df, on=['Sequence ID', 'Primer Rank'],
                                      how='left')
    return merged_tabix_df

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Unit tester for core.py

Dependencies:
 - python3.6+
   - unittest
   - pandas >= 0.22.0
"""

import unittest
import pandas as pd
from primer_tk import core

class TestCoreSV(unittest.TestCase):
    """
    Test all core functions for SV pipeline.
    """
    def setUp(self):
        """
        Inputs for each function.
        """
        pass
    def tearDown(self):
        pass

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
Unit tester for genome_iterator.py

Python version: Python 3.6.8 :: Anaconda, Inc.
Date: 02/28/2019
Dependencies:
 - unittest
 - pandas>=0.22.0
"""

import unittest
import pandas as pd
from primer_tk import genome_iterator as gi

class TestGenomeIterator(unittest.TestCase):
    """
    Subclass of unittest to test genome_iterator.py
    """
    def setUp(self):
        self.ref_genome = "test/data/test_standard.fa"
        self.genome = gi.genome_iterator(self.ref_genome)
        self.test_input = 'test/data/input_standard.csv'
        self.test_input2 = 'test/data/input_standard.txt'
        self.test_input3 = 'test/data/input_standard.fa'
        #self.parser = gi.get_args_iterator([])

    def test_genome_iterator(self):
        """
        Asserts object output by genome_iterator function has length of 2 (header, seq)
        """
        self.assertEqual(len(self.genome[0]), 2)

    def test_create_dataframe_csv(self):
        """
        Asserts that a pd.DataFrame is returned from function.
        """
        self.assertTrue(isinstance(gi.create_dataframe_csv(self.test_input), pd.DataFrame))

    def test_create_dataframe_txt(self):
        """
        Assets that a pd.DataFrame is returned from function.
        """
        self.assertTrue(isinstance(gi.create_dataframe_txt(self.test_input2), pd.DataFrame))

    def test_file_extension_success(self):
        """
        Asserts that a pd.DataFrame is returned only if file extension matches.
        """
        self.assertTrue(isinstance(gi.file_extension(self.test_input2), pd.DataFrame))

    def test_file_extension_failure(self):
        """
        Asserts that code exits with warning message if file extension is incorrect.
        """
        with self.assertRaises(SystemExit) as code_message:
            gi.file_extension(self.test_input3)
        self.assertEqual(code_message.exception.code,
                         "Wrong File Format, should be .txt or .csv")

    def test_match_chr_to_genome(self):
        """
        Asserts that a pd.DataFrame object is returned from function.
        """
        dataframe = gi.file_extension(self.test_input)
        self.assertTrue(isinstance(gi.match_chr_to_genome(dataframe, self.genome), pd.DataFrame))


    def test_flanking_regions_fasta(self):
        """
        Asserts that return object (list of tuples) first item is sample.
        """
        dataframe = gi.file_extension(self.test_input)
        self.assertTrue((gi.create_flanking_regions_fasta\
                             (self.genome, dataframe, 200)[0][0]).startswith('sample'))

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
Unit tester for genome_iterator_sv.py

Python version: Python 3.6.8 :: Anaconda, Inc.
Date: 02/28/2019
Dependencies:
 - unittest
 - pandas >= 0.22.0
"""

import unittest
import pandas as pd
from primer_tk import genome_iterator_sv as gi

class TestGenomeIterator(unittest.TestCase):
    """
    Subclass of unittest to test genome_iterator_sv.py
    """
    def setUp(self):
        self.ref_genome = "test/data/test_sv.fa"
        self.genome = gi.genome_iterator(self.ref_genome)
        self.test_input = 'test/data/input_sv.csv'
        self.test_input2 = 'test/data/input_sv.txt'
        self.test_input3 = 'test/data/input_sv.fa'
        self.insertion_genome = gi.genome_iterator('test/data/insertion_test.fa')
        self.insertion_input1 = 'test/data/insertion_input.csv'
        self.insertion_input2 = 'test/data/insertion_input.txt'

    def test_genome_iterator(self):
        """
        Asserts object output by genome_iterator function has length of 2 (header, seq)
        """
        self.assertEqual(len(self.genome[0]), 2)

    def test_create_dataframe_csv(self):
        """
        Asserts that a pd.DataFrame is returned from function.
        """
        self.assertTrue(isinstance(gi.create_dataframe_csv(self.test_input), pd.DataFrame))

    def test_create_dataframe_txt(self):
        """
        Asserts that a pd.DataFrame is returned from function.
        """
        self.assertTrue(isinstance(gi.create_dataframe_txt(self.test_input2), pd.DataFrame))

    def test_create_dataframe_insertion_csv(self):
        """
        Asserts that proper insertion dataframe is returned from function.
        """
        self.assertEqual(len(gi.create_dataframe_insertion_csv(self.insertion_input1)),2)

    def test_create_dataframe_insertion_txt(self):
        """
        Asserts that proper insertion dataframe is returned from function.
        """
        self.assertEqual(len(gi.create_dataframe_insertion_txt(self.insertion_input2)),2)

    def test_file_extension_insertion_success(self):
        """
        Asserts that proper df is created when sv type is insertion.
        """
        self.assertEqual(len(gi.file_extension(self.insertion_input1, 'insertion')),2)

    def test_file_extension_deletion_success(self):
        """
        Asserts that a pd.DataFrame is returned only if file extension matches.
        """
        self.assertTrue(isinstance(gi.file_extension(self.test_input2, 'deletion'), pd.DataFrame))

    def test_file_extension_inversion_success(self):
        """
        Asserts that a pd.DataFrame is returned only if file extension matches.
        """
        self.assertTrue(isinstance(gi.file_extension(self.test_input2, 'inversion'), pd.DataFrame))

    def test_file_extension_failure(self):
        """
        Asserts that code exits with warning message if file extension or sv is incorrect.
        """
        with self.assertRaises(SystemExit) as code_message:
            gi.file_extension(self.test_input3, 'deletion')
        self.assertEqual(code_message.exception.code,
                         "Wrong File Format, should be .txt (tab) or .csv (comma), or check sv type.")

    def test_match_chr_to_genome(self):
        """
        Asserts that a pd.DataFrame object is returned from function.
        chr format is mismatched between genome and input file, checks for conversion.
        """
        dataframe = gi.file_extension(self.test_input2, 'deletion')
        self.assertTrue(isinstance(gi.match_chr_to_genome(dataframe, self.genome, 'deletion'), pd.DataFrame))

    def test_match_chr_to_genome_insertion(self):
        """
        Asserts that input file with format chr1 is converted to 1 to match ref genome.
        """
        dataframe = gi.file_extension(self.insertion_input1, 'insertion')
        self.assertFalse(gi.match_chr_to_genome(dataframe, self.insertion_genome, 'insertion')\
                             ["ChrNorm"].str.contains("chr").any())

    def test_flanking_regions_fasta_deletion(self):
        """
        Asserts that return object (list of tuples) first item is sample.
        """
        dataframe = gi.file_extension(self.test_input, 'deletion')
        self.assertTrue((gi.flanking_regions_fasta_deletion\
                             (self.genome, dataframe, 100)[0][0]).startswith('sample'))

    def test_flanking_regions_fasta_insertion(self):
        """
        Asserts that insertions are handled properly for both forward and reverse strand insertions.
        """
        dataframe = gi.file_extension(self.insertion_input2, 'insertion')
        flanking = gi.flanking_region_fasta_insertion(self.insertion_genome, dataframe, 5)
        self.assertTrue(flanking[0][1] == 'AAAAACCCCC')
        self.assertTrue(flanking[1][1] == 'AAAAAGAATT')
        self.assertTrue(flanking[2][1] == 'CTTAATTTTT')
        self.assertTrue(flanking[3][1] == 'GGGGGTTTTT')
    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

"""
Modules required for program.
 - python3.6+
    - pandas>=0.22.0
    - argparse
    - primer_cross_hyb
    - sequence_info
"""

import unittest
from io import StringIO
import argparse
from primer_tk.mp_class import MissingPrimers
from primer_tk.mp_class import create_df
from primer_tk import primer_cross_hyb as pch

class TestPrimerCrossHyb(unittest.TestCase):
    """
    Subclass of unittest to test primer_cross_hyb.py
    """
    def setUp(self):
        """
        Setup for tests, need to generate df as all scripts in this operate
        on a dataframe. self.primer_df is called twice to drop NA.
        """
        self.dump = "test/data/primer_dump_standard.txt"
        prim_list_0 = MissingPrimers(self.dump, 0).samp_primer_info
        prim_list_1 = MissingPrimers(self.dump, 1).samp_primer_info
        prim_list_2 = MissingPrimers(self.dump, 2).samp_primer_info
        prim_list_3 = MissingPrimers(self.dump, 3).samp_primer_info
        prim_list_4 = MissingPrimers(self.dump, 4).samp_primer_info
        self.primer_df = create_df([prim_list_0, prim_list_1, prim_list_2,
                                    prim_list_3, prim_list_4])
        self.primer_df = self.primer_df.loc[~(self.primer_df['Primer Left Seq'] == 'NA')]
        self.pa = 50
        self.fp_len = pch.get_fprimer_percent_aln(self.primer_df['Primer Left Seq'], self.pa)
        self.pd_compare_1_2 =  pch.primer_dimer_local(self.fp_len, self.primer_df['Sequence ID'],
                                                      self.primer_df['Primer Left Seq'],
                                                      self.primer_df['Primer Right Seq'])



    def test_get_fprimer_percent_aln(self):
        """
        Tests to ensure that the proper forward primer length is returned for
        the given percent alignment.
        primer lengths input = [22, 22, 22, 22, 21, 22, 24, 22, 22, 22, 22, 24]
        Should return = [11, 11, 11, 11, 10, 11, 12, 11, 11, 11, 11, 12]
        """
        self.assertEqual(self.fp_len, [11, 11, 11, 11, 10, 11, 12, 11, 11, 11, 11, 12])


    def test_primer_dimer_local(self):
        """
        For percent alignment == 50, this set of primers should contain 17 dimers in
        forward primer vs reverse primer comparison.
        """
        self.assertEqual(len(list(self.pd_compare_1_2)), 17)

    def test_list_from_gen(self):
        """
        Tests to ensure list is generated from primer_dimer_local output.
        """
        self.assertTrue(isinstance(pch.list_from_gen(self.pd_compare_1_2), list))

    def test_p_list_formatter(self):
        """
        Tests to ensure that output list is formatted properly.
        """
        pd_list = pch.list_from_gen(self.pd_compare_1_2)
        formatted = pch.p_list_formatter(pd_list)
        self.assertFalse(any('\\n' in x for x in formatted))

    def test_dimer_true(self):
        """
        Tests to ensure that exactly 3 dimers are found between forward and reverse primers.
        """
        pd_list = pch.list_from_gen(self.pd_compare_1_2)
        formatted = pch.p_list_formatter(pd_list)
        self.primer_df['Dimers1_2F'] = pch.dimer_true(self.primer_df, 2, formatted)
        self.assertEqual((self.primer_df['Dimers1_2F'][self.primer_df['Dimers1_2F'] == True].count()), 3)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

def primer3_input(header, sequence, args):
    """ Generates a primer3 input file with specified arguments.
    Args:
        header (string): The primer header string
        sequence (string): the sequence used to design primers
        args (Namespace): Arparse results.
    Returns:
        primer3_input_string (string): The string to write to the primer3 input file.
    """

    primer3_input_string = "SEQUENCE_ID="+header+'\n'\
    +"SEQUENCE_TEMPLATE="+sequence+'\n'\
    +"SEQUENCE_TARGET=%s" %args.sequence_target+'\n'\
    +"PRIMER_FIRST_BASE_INDEX=1"+'\n'\
    +"PRIMER_TASK=pick_detection_primers"+'\n'\
    +"PRIMER_MIN_THREE_PRIME_DISTANCE=3"+'\n'\
    +"PRIMER_MAX_LIBRARY_MISPRIMING=12.00"+'\n'\
    +"PRIMER_PAIR_MAX_LIBRARY_MISPRIMING=20.00"+'\n'\
    +"PRIMER_PRODUCT_SIZE_RANGE=%s" %args.product_size_range+'\n'\
    +"PRIMER_MAX_END_STABILITY=9.0"+'\n'\
    +"PRIMER_MAX_SELF_ANY_TH=45.00"+'\n'\
    +"PRIMER_MAX_SELF_END_TH=35.00"+'\n'\
    +"PRIMER_PAIR_MAX_COMPL_ANY_TH=45.00"+'\n'\
    +"PRIMER_PAIR_MAX_COMPL_END_TH=35.00"+'\n'\
    +"PRIMER_MAX_HAIRPIN_TH=24.00"+'\n'\
    +"PRIMER_MAX_TEMPLATE_MISPRIMING_TH=40.00"+'\n'\
    +"PRIMER_PAIR_MAX_TEMPLATE_MISPRIMING_TH=70.00"+'\n'\
    +"PRIMER_TM_FORMULA=1"+'\n'\
    +"PRIMER_SALT_CORRECTIONS=1"+'\n'\
    +"PRIMER_SALT_MONOVALENT=50.0"+'\n'\
    +"PRIMER_INTERNAL_SALT_MONOVALENT=50.0"+'\n'\
    +"PRIMER_SALT_DIVALENT=1.5"+'\n'\
    +"PRIMER_INTERNAL_SALT_DIVALENT=1.5"+'\n'\
    +"PRIMER_DNTP_CONC=0.6"+'\n'\
    +"PRIMER_INTERNAL_DNTP_CONC=0.6"+'\n'\
    +"PRIMER_DNA_CONC=50.0"+'\n'\
    +"PRIMER_INTERNAL_DNA_CONC=50.0"+'\n'\
    +"PRIMER_THERMODYNAMIC_OLIGO_ALIGNMENT=1"+'\n'\
    +"PRIMER_THERMODYNAMIC_TEMPLATE_ALIGNMENT=1"+'\n'\
    +"PRIMER_THERMODYNAMIC_PARAMETERS_PATH=%s" %args.thermopath+'\n'\
    +"PRIMER_PICK_LEFT_PRIMER=1"+'\n'\
    +"PRIMER_PICK_RIGHT_PRIMER=1"+'\n'\
    +"PRIMER_PICK_INTERNAL_OLIGO=1"+'\n'\
    +"PRIMER_MAX_POLY_X=3"+'\n'\
    +"PRIMER_LEFT_NUM_RETURNED=5"+'\n'\
    +"PRIMER_RIGHT_NUM_RETURNED=5"+'\n'\
    +"PRIMER_OPT_SIZE=%s" %args.primer_opt_size+'\n'\
    +"PRIMER_MIN_SIZE=%s" %args.primer_min_size+'\n'\
    +"PRIMER_MAX_SIZE=%s" %args.primer_max_size+'\n'\
    +"PRIMER_MIN_TM=%s" %args.primer_min_tm+'\n'\
    +"PRIMER_OPT_TM=%s" %args.primer_opt_tm+'\n'\
    +"PRIMER_MAX_TM=%s" %args.primer_max_tm+'\n'\
    +"PRIMER_MAX_NS_ACCEPTED=1"+'\n'\
    +"PRIMER_NUM_RETURN=5"+'\n'\
    +"P3_FILE_FLAG=1"+'\n'\
    +"PRIMER_EXPLAIN_FLAG=1"+'\n'\
    +"PRIMER_MISPRIMING_LIBRARY=%s" %args.mispriming+'\n'\
    +"PRIMER_MIN_GC=%s" %args.primer_min_gc+'\n'\
    +"PRIMER_OPT_GC_PERCENT=%s" %args.primer_opt_gc+'\n'\
    +"PRIMER_MAX_GC=%s" %args.primer_max_gc+'\n'\
    +"PRIMER_PAIR_MAX_DIFF_TM=3"+'\n'\
    +"="+'\n'
    return primer3_input_string



#!/usr/bin/env python3
"""
Unit tester for primer_tabix.py

Python version: Python 3.6.8 :: Anaconda, Inc.
Dependencies:
 - unittest
 - pandas >= 0.22.0
 - pysam >= 0.15.2
"""

import unittest
import pandas as pd
from pysam import VariantFile
from primer_tk import primer_tabix as pt

class TestPrimerTabix(unittest.TestCase):
    """
    Subclass of unittest to test primer_tabix.py
    """
    def setUp(self):
        self.primer_output = "test/data/top_ranked_final_primers_standard.csv"
        self.vcf_test = 'test/data/primer_test_vcf.csv'
        self.pinfo = pt.create_tabix_df(self.primer_output)
        self.pvcf = pt.create_tabix_df(self.vcf_test)
        self.vcf = 'test/data/smallvcf.vcf.gz'
        self.vcf_in = VariantFile(self.vcf)

    def test_create_tabix_df(self):
        """
        Test dataframe creation.
        """
        self.assertTrue(isinstance(pt.create_tabix_df(self.primer_output), pd.DataFrame))

    def test_primer_range_left(self):
        """
        Test the range info is added to the dataframe.
        """
        self.assertEqual(len(pt.primer_range_left(self.pinfo['Sequence ID'], self.pinfo['Primer Rank'],
                                                  self.pinfo['Chromosome'], self.pinfo['Primer Left Seq'],
                                                  self.pinfo['Position1']).columns), 6)
    def test_primer_range_right(self):
        """
        Test the range info is added to the dataframe.
        """
        self.assertEqual(len(pt.primer_range_left(self.pinfo['Sequence ID'], self.pinfo['Primer Rank'],
                                                  self.pinfo['Chromosome'], self.pinfo['Primer Right Seq'],
                                                  self.pinfo['Position2']).columns), 6)

    def test_match_pinfo_to_vcf(self):
        """
        Normalizes the chr string to the reference vcf.
        """
        self.assertFalse(pt.match_pinfo_to_vcf(self.pinfo, self.vcf_in)['Chromosome'].str.contains("chr").any())

    def test_tabix_fetch(self):
        """
        Fetches snp info and assigns to primers.
        """
        left = pt.primer_range_left(self.pvcf['Sequence ID'], self.pvcf['Primer Rank'],
                                    self.pvcf['Chromosome'], self.pvcf['Primer Left Seq'],
                                    self.pvcf['Position1'])
        normalized = pt.match_pinfo_to_vcf(left, self.vcf_in)
        left_snps = pt.tabix_fetch(normalized['Sequence ID'], normalized['Primer Rank'],
                                   normalized['Chromosome'], normalized['Position1'],
                                   normalized['Position2'], self.vcf_in)
        self.assertEqual(len(left_snps), 20)

    def test_tabix_results_to_df(self):
        """
        Generates a pd.DataFrame from the tabix results.
        """
        left = pt.primer_range_left(self.pvcf['Sequence ID'], self.pvcf['Primer Rank'],
                                    self.pvcf['Chromosome'], self.pvcf['Primer Left Seq'],
                                    self.pvcf['Position1'])
        normalized = pt.match_pinfo_to_vcf(left, self.vcf_in)
        left_snps = pt.tabix_fetch(normalized['Sequence ID'], normalized['Primer Rank'],
                                   normalized['Chromosome'], normalized['Position1'],
                                   normalized['Position2'], self.vcf_in)
        left_df = pt.tabix_results_to_df(left_snps, "L", "Left SNP Count")
        self.assertEqual(left_df['Left SNP Count'][0], 10)

    def test_merge_left_right(self):
        """
        Merge the left and right SNP dataframes.
        """
        left = pt.primer_range_left(self.pvcf['Sequence ID'], self.pvcf['Primer Rank'],
                                    self.pvcf['Chromosome'], self.pvcf['Primer Left Seq'],
                                    self.pvcf['Position1'])
        normalized = pt.match_pinfo_to_vcf(left, self.vcf_in)
        left_snps = pt.tabix_fetch(normalized['Sequence ID'], normalized['Primer Rank'],
                                   normalized['Chromosome'], normalized['Position1'],
                                   normalized['Position2'], self.vcf_in)
        left_df = pt.tabix_results_to_df(left_snps, "L", "Left SNP Count")
        right = pt.primer_range_right(self.pvcf['Sequence ID'], self.pvcf['Primer Rank'],
                                      self.pvcf['Chromosome'], self.pvcf['Primer Right Seq'],
                                      self.pvcf['Position2'])
        normalized = pt.match_pinfo_to_vcf(right, self.vcf_in)
        right_snps = pt.tabix_fetch(normalized['Sequence ID'], normalized['Primer Rank'],
                                    normalized['Chromosome'], normalized['Position1'],
                                    normalized['Position2'], self.vcf_in)
        right_df = pt.tabix_results_to_df(left_snps, "R", "Right SNP Count")
        merged_df = pt.merge_left_right(left_df, right_df, self.pvcf)
        self.assertTrue('Left SNP Count' and 'Right SNP Count' in merged_df.columns)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
""" Core modules """

import os
from pysam import VariantFile
from primer_tk import genome_iterator as gi
from primer_tk import analyze_pcr_output as ap
from primer_tk import analyze_pcr_output_sv as apv
from primer_tk.mp_class import MissingPrimers as mp
from primer_tk.mp_class import create_df as cd
from primer_tk.mp_class_sv import MissingPrimers as mpv
from primer_tk.mp_class_sv import create_df as cdv
from primer_tk import utils
import primer_tk.primer_cross_hyb as pch
from primer_tk import primer_tabix as pt
from primer_tk import genome_iterator_sv as giv


def iterator_sv(args):
    """ Use an input regions file with SV positions to pull
        down flanking sequence on both sides of SV to generate
        primers upstream and downstream of the SV.

    Args:
        args (Namespace): Argparse results.

    Returns: None
    """

    dataset_name = os.path.splitext(str(args.regions_file))[0]
    genome = giv.genome_iterator(args.ref_genome)
    # 2) create dataframe from input regions file
    if args.sv in ('deletion', 'inversion'):
        small_regions = giv.file_extension(args.regions_file, args.sv)
    # 3) ensure proper proper number of columns in dataframe
        assert len(list(small_regions)) == 5, "DataFrame contains more/less than 5 columns...\
                                           Improper format."
    # 4) format dataframe "chr" column to match reference genome
        small_regions = giv.match_chr_to_genome(small_regions, genome, args.sv)
    # 5) generate flanking regions fasta based on position in input file
        flanking = open("flanking_regions.%s.fasta" % dataset_name, 'w')
        if args.sv == 'deletion':
            flank_data = giv.flanking_regions_fasta_deletion(genome, small_regions, args.flanking_region_size)
            primer3_in = open("primer3_input.%s.txt" % dataset_name, 'w')
            for head, seq in flank_data:
                flanking.write(">"+head+'\n'+seq+'\n')
            # 6) generate primer3 input file
                primer3_in.write(utils.primer3_input(head, seq, args))
        elif args.sv == 'inversion':
            flank_data = giv.flanking_regions_fasta_inversion(genome, small_regions, args.flanking_region_size)
            primer3_in = open("primer3_input.%s.txt" % dataset_name, 'w')
            for head, seq in flank_data:
                flanking.write(">"+head+'\n'+seq+'\n')
            # 6) generate primer3 input file
                primer3_in.write(utils.primer3_input(head, seq, args))
        flanking.close()
        primer3_in.close()

    elif args.sv == 'insertion':
        small_regions = giv.file_extension(args.regions_file, args.sv)
        assert len(list(small_regions)) == 9, "DataFrame contains more/less than 9 columns... Exiting."
        small_regions = giv.match_chr_to_genome(small_regions, genome, args.sv)
        flanking = open("flanking_regions.%s.fasta" %dataset_name, 'w')
        flank_data = giv.flanking_region_fasta_insertion(genome, small_regions, args.flanking_region_size)
        primer3_in = open("primer3_input.%s.txt" % dataset_name, 'w')
        for head, seq in flank_data:
            flanking.write(">"+head+'\n'+seq+'\n')
            primer3_in.write(utils.primer3_input(head, seq, args))
    flanking.close()
    primer3_in.close()

def iterator(args):
    """
    Use an input regions file with specific region of interest\
    to design primers around, then run primer3.

    Args:
        args (Namespace): Argparse object or None.

    Returns: None
    """

    dataset_name = os.path.splitext(str(args.regions_file))[0]
    # 1) create genome tuple from provided reference
    genome = gi.genome_iterator(args.ref_genome)
    # 2) create dataframe from input regions file
    small_regions = gi.file_extension(args.regions_file)
    # 3) ensure proper proper number of columns in dataframe
    assert len(list(small_regions)) == 4, "DataFrame contains more than 4 columns...\
                                           Improper format."
    # 4) format dataframe "chr" column to match reference genome
    small_regions = gi.match_chr_to_genome(small_regions, genome)
    # 5) generate flanking regions fasta based on position in input file
    flanking = open("flanking_regions.%s.fasta" % dataset_name, 'w')
    flank_data = gi.create_flanking_regions_fasta(genome, small_regions, args.flanking_region_size)
    primer3_in = open("primer3_input.%s.txt" % dataset_name, 'w')
    for head, seq in flank_data:
        flanking.write(">"+head+'\n'+seq+'\n')
    # 6) generate primer3 input file
        primer3_in.write(utils.primer3_input(head, seq, args))
    flanking.close()
    primer3_in.close()

def pre(args):
    """ Function for all steps leading up to PCR. """
    # 1) Initialize primer lists by rank for each sample
    prim_list_0 = mp(args.dump, 0).samp_primer_info
    prim_list_1 = mp(args.dump, 1).samp_primer_info
    prim_list_2 = mp(args.dump, 2).samp_primer_info
    prim_list_3 = mp(args.dump, 3).samp_primer_info
    prim_list_4 = mp(args.dump, 4).samp_primer_info
    # 2) Generate the output df
    primer_df = cd([prim_list_0, prim_list_1, prim_list_2,
                    prim_list_3, prim_list_4])
    # 3) Generate csv output
    primer_df = primer_df.loc[~(primer_df['Primer Left Seq'] == 'NA')]
    primer_df.to_csv(args.outfile, index=False)
    primer_df_standard = primer_df.copy()
    # 5) Get length of forward primers for percent alignment check
    if args.pcr == 'multiplex':
        fp_len = pch.get_fprimer_percent_aln(primer_df['Primer Left Seq'], int(args.percent_alignment))
    # 6) Generate primer dimer pairs for all vs all input
        primer1_2_compare = pch.list_from_gen(pch.primer_dimer_local(fp_len, primer_df['Sequence ID'],
                                                                     primer_df['Primer Left Seq'],
                                                                     primer_df['Primer Right Seq']))
        primer2_1_compare = pch.list_from_gen(pch.primer_dimer_local(fp_len, primer_df['Sequence ID'],
                                                                     primer_df['Primer Right Seq'],
                                                                     primer_df['Primer Left Seq']))
        primer1_1_compare = pch.list_from_gen(pch.primer_dimer_local(fp_len, primer_df['Sequence ID'],
                                                                     primer_df['Primer Left Seq'],
                                                                     primer_df['Primer Left Seq']))
        primer2_2_compare = pch.list_from_gen(pch.primer_dimer_local(fp_len, primer_df['Sequence ID'],
                                                                     primer_df['Primer Right Seq'],
                                                                     primer_df['Primer Right Seq']))
    # 7) Reformat output, Biopython + Class writes ugly output.
        primer1_2_dimers = pch.p_list_formatter(primer1_2_compare)
        primer2_1_dimers = pch.p_list_formatter(primer2_1_compare)
        primer1_1_dimers = pch.p_list_formatter(primer1_1_compare)
        primer2_2_dimers = pch.p_list_formatter(primer2_2_compare)
    # 8) Write output of all primer pairs that form dimers.
        pd_file = open('Primer_Dimers.txt', 'w')
        pd_file.write('#This is a list of possible primer dimers based on the input complementarity.\n')
        pd_file.write('#Comparison of forward primers with all reverse primers...\
                       \n#Sequence ID \t \t Complementarity Score\n')
        for seq in primer1_2_dimers:
            pd_file.write(str(seq) + '\n')
            pd_file.write('#Comparison of reverse primers with all forward primers...\n')
        for seq in primer2_1_dimers:
            pd_file.write(str(seq) + '\n')
            pd_file.write('#Comparison of forward primers with all other forward primers...\n')
        for seq in primer1_1_dimers:
            pd_file.write(str(seq) + '\n')
            pd_file.write('#Comparison of revers primers with all other reverse primers...\n')
        for seq in primer2_2_dimers:
            pd_file.write(str(seq) + '\n')
        pd_file.close()
    # 9) Searche dataframe for dimers in list. If present, marked with True boolean in new column.
    # Else, False. Used for filtering in next step.
        primer_df['Dimers1_2F'] = pch.dimer_true(primer_df, 2, primer1_2_dimers)
        primer_df['Dimers2_1F'] = pch.dimer_true(primer_df, 2, primer2_1_dimers)
        primer_df['Dimers1_1F'] = pch.dimer_true(primer_df, 2, primer1_1_dimers)
        primer_df['Dimers2_2F'] = pch.dimer_true(primer_df, 2, primer2_2_dimers)
        primer_df['Dimers1_2R'] = pch.dimer_true(primer_df, 3, primer1_2_dimers)
        primer_df['Dimers2_1R'] = pch.dimer_true(primer_df, 3, primer2_1_dimers)
        primer_df['Dimers1_1R'] = pch.dimer_true(primer_df, 3, primer1_1_dimers)
        primer_df['Dimers2_2R'] = pch.dimer_true(primer_df, 3, primer2_2_dimers)
    # 10) Search for true statements, True statements indicate dimer duo. Only keep non-dimers
        df_bool = (primer_df.loc[~(primer_df['Dimers1_2F'] | primer_df['Dimers2_1F']\
                                   | primer_df['Dimers1_1F'] | primer_df['Dimers2_2F']\
                                   | primer_df['Dimers1_2R'] | primer_df['Dimers2_1R']\
                                   | primer_df['Dimers1_1R'] | primer_df['Dimers2_2R'] == True)])
    # 11) Write output to csv
        df_bool.to_csv(args.no_dimer, index=False)
        pch.all_vs_all_pcr(df_bool, args.multiplex_pcr_file)
    elif args.pcr == 'standard':
        pch.standard_pcr(primer_df_standard, args.standard_pcr_file)
    else:
        print("Please select pcr setup")

def pre_sv(args):
    """
    Function for all steps leading up to PCR.
    """
    # 1) Initialize primer lists by rank for each sample
    prim_list_0 = mpv(args.dump, 0).samp_primer_info
    prim_list_1 = mpv(args.dump, 1).samp_primer_info
    prim_list_2 = mpv(args.dump, 2).samp_primer_info
    prim_list_3 = mpv(args.dump, 3).samp_primer_info
    prim_list_4 = mpv(args.dump, 4).samp_primer_info
    # 2) Generate the output df
    primer_df = cdv([prim_list_0, prim_list_1, prim_list_2,
                     prim_list_3, prim_list_4])
    # 3) Generate csv output
    primer_df = primer_df.loc[~(primer_df['Primer Left Seq'] == 'NA')]
    primer_df.to_csv(args.outfile, index=False)
    # 4) create standard pcr input
    pch.standard_pcr(primer_df, args.pcrfile)

def post(args):
    """
    Generates pcr analysis dataframes and applies primer filtering based on
    off-target amplification. Then compares good primers to initial primer
    list to find which primer pair was generated and top ranking.
    Finally, produces easy to use IDT order sheet in plate format (standard PCR only).
    """
    # 2) Generate seqs and headers lists
    seqs, headers = ap.fasta_parser(args.pcrfile)
    # 3) Calculate GC of each PCR product and store in list
    gc_list = ap.gc_percent_seqs(seqs)
    # 4) Split up the header line and get no_chrom_list
    split_headers, no_chrom = ap.split_headers_list(headers)
    # 5) Generate chrom list
    chrom_list = ap.chr_split_list(split_headers)
    # 6) Get split positions list
    pos_split = ap.pos_split_list(chrom_list)
    # 7) Need to delete positions after extracting positions (ugly)
    for item in chrom_list:
        del item[-1]
    # 7) Get name and pos split list
    name_pos_list = ap.split_name_pos(no_chrom)
    # 8) Merge all these lists for dataframe
    merged_list = ap.merge_info(chrom_list, pos_split, name_pos_list, no_chrom)
    all_pcr_df, good_primers_df, bad_primers_df = ap.generate_pcr_df(merged_list, gc_list)
    # 9) Output file generation
    all_pcr_df.to_csv(args.pcr_product_info, index=False)
    # 10) Merge good primers df with toal primers df
    merged_df = ap.merge_good_total(good_primers_df, args.total_primers)
    # 11) Keep only primers which match bw good and total primers
    filtered_df = ap.filter_merged(merged_df, args.off_target_max)
    filtered_df.to_csv(args.all_primer_info, index=False)
    # 12) Output only top ranked final primers after filter
    top_ranked_df = ap.top_ranked_final_primers(filtered_df)
    top_ranked_df.to_csv(args.top_final_primers, index=False)
    plate_order_f, plate_order_r = ap.to_order_plate(top_ranked_df)
    plate_order_f.to_csv(args.plate_basename+'_F.csv', index=False)
    plate_order_r.to_csv(args.plate_basename+'_R.csv', index=False)

def post_sv(args):
    """
    Generates pcr analysis dataframes and applies primer filtering based on
    off-target amplification. Then compares good primers to initial primer
    list to find which primer pair was generated and top ranking.
    Finally, produces easy to use IDT order sheet in plate format (standard PCR only).
    """
    # 2) Generate seqs and headers lists
    seqs, headers = apv.fasta_parser(args.flank_file)
    # 3) Calculate GC of each PCR product and store in list
    positions_to_compare = apv.amp_header_region(args.total_primers)
    sliced_seqs = apv.get_gc_region(seqs, headers, positions_to_compare)
    gc_calc = apv.calc_gc(sliced_seqs)
    merged_df = apv.merge_dfs(gc_calc, args.total_primers, seqs)
    merged_df.to_csv(args.all_final_primers, index=False)
    merged_df.drop_duplicates('Sequence ID', keep='first', inplace=True)
    merged_df.to_csv(args.top_final_primers, index=False)
    plate_order_f, plate_order_r = apv.to_order_plate(merged_df)
    plate_order_f.to_csv(args.plate_basename+'_F.csv', index=False)
    plate_order_r.to_csv(args.plate_basename+'_R.csv', index=False)

def tabix(args):
    """
    Annotates primers with SNP information.
    """
    vcf_in = VariantFile(args.vcf)
    p_info = pt.create_tabix_df(args.p_info)
    p_left = pt.primer_range_left(p_info["Sequence ID"],
                                  p_info["Primer Rank"],
                                  p_info["Chromosome"],
                                  p_info["Primer Left Seq"],
                                  p_info["Position1"])
    p_right = pt.primer_range_right(p_info["Sequence ID"],
                                    p_info["Primer Rank"],
                                    p_info["Chromosome"],
                                    p_info["Primer Right Seq"],
                                    p_info["Position2"])
    pn_left = pt.match_pinfo_to_vcf(p_left, vcf_in)
    pn_right = pt.match_pinfo_to_vcf(p_right, vcf_in)
    left_snps = pt.tabix_fetch(pn_left["Sequence ID"],
                               pn_left["Primer Rank"],
                               pn_left["Chromosome"],
                               pn_left["Position1"],
                               pn_left["Position2"],
                               vcf_in)
    right_snps = pt.tabix_fetch(pn_right["Sequence ID"],
                                pn_right["Primer Rank"],
                                pn_right["Chromosome"],
                                pn_right["Position1"],
                                pn_right["Position2"],
                                vcf_in)
    left_df = pt.tabix_results_to_df(left_snps, "L", "Left SNP Count")
    right_df = pt.tabix_results_to_df(right_snps, "R", "Right SNP Count")
    merged_df = pt.merge_left_right(left_df, right_df, p_info)
    merged_df.to_csv(args.output, index=False)



import setuptools

setuptools.setup(
    name='primer_tk',
    version='1.0.0',
    scripts=['./scripts/primer_tk'],
    author="Dennis Kennetz",
    author_email="dennis.kennetz@stjude.org",
    description="A toolkit to design primers in multiplex pools and around SVs.",
    packages=['primer_tk', 'primer_tk.tests'],
    package_dir={'primer_tk': 'src/primer_tk', 'primer_tk.tests': 'test/python_tests'},
    package_data={'test': [
        'data/*']},
    install_requires=[
        'setuptools',
        'pandas >= 0.22.0',
        'numpy >= 1.16.0',
        'biopython >= 1.70',
        'pysam==0.15.2'
    ],
    python_requires='>=3.5.*',
    test_suite='test',
    tests_require=['unittest', 'coverage'],
    zip_safe=True,
    license='Apache2.0',
    url = 'https://github.com/stjude/PrimerTK',
    download_url = 'https://github.com/stjude/PrimerTK/archive/1.0.0.tar.gz',
    classifiers=[
        "Programming Language :: Python :: 3",
        'Environment :: Console',
        'Natural Language :: English',
        'Operating System :: OS Independent'
    ],
)



#!/usr/bin/env python3

"""
Modules required for program.
 - python3.6+
   - pandas >= 0.22.0
   - sequence_info
"""

import unittest
import pandas as pd
from primer_tk import analyze_pcr_output as ap

class TestAnalyzePcrOutput(unittest.TestCase):
    """
    Subclass of unittest to test analyze_pcr_output.py
    """
    def setUp(self):
        self.pcr_file = 'test/data/pcr_standard.fa'
        self.primer_file = 'test/data/total_primers_standard.csv'
        self.seqs, self.headers = ap.fasta_parser(self.pcr_file)
        self.gc_list = ap.gc_percent_seqs(self.seqs)
        self.split_headers, self.no_chrom = ap.split_headers_list(self.headers)
        self.chr_list = ap.chr_split_list(self.split_headers)
        self.pos_list = ap.pos_split_list(self.chr_list)
        self.name_pos_split_list = ap.split_name_pos(self.no_chrom)
        for item in self.chr_list:
            del item[-1]
        self.merged = ap.merge_info(self.chr_list, self.pos_list,
                                    self.name_pos_split_list,
                                    self.no_chrom)
        self.pcr_df, self.good_primers, self.bad_primers = ap.generate_pcr_df(self.merged,
                                                                              self.gc_list)
        self.merged_df = ap.merge_good_total(self.good_primers, self.primer_file)
        self.filtered = ap.filter_merged(self.merged_df, 1)

    def test_fasta_parser(self):
        """
        Ensure that seq and header are proper length.
        """
        seqs, headers = ap.fasta_parser(self.pcr_file)
        self.assertTrue(len(seqs[0][0]) == 306)
        self.assertTrue(len(headers[0]) == 98)

    def test_gc_percent_seqs(self):
        """
        Calcs gc percent of list of seqs
        """
        self.assertEqual(int(ap.gc_percent_seqs(self.seqs)[0]), 49)

    def test_split_headers_list(self):
        """
        Split header information and return tuple of parsed header info.
        """
        self.assertTrue(len(ap.split_headers_list(self.headers)[0][0]) >
                        len(ap.split_headers_list(self.headers)[1][0]))

    def test_chr_split_list(self):
        """
        Parses chrm and position out of headers list.
        """
        self.assertEqual(ap.chr_split_list(self.split_headers)[0][0], '1')

    def test_split_name_pos(self):
        """
        Gets the expected position of the snp.
        """
        self.assertEqual(ap.split_name_pos(self.no_chrom)[0][0], '15000000')

    def test_merge_info(self):
        """
        Merges all the info extracted from the fasta files.
        """
        self.assertEqual(len(ap.merge_info(self.chr_list, self.pos_list,
                                       self.name_pos_split_list,
                                           self.no_chrom)[0]), 8)

    def test_generate_pcr_df_off_target(self):
        """
        Creates a df from all info.
        """
        pcr_df, good_primers, bad_primers = ap.generate_pcr_df(self.merged, self.gc_list)
        self.assertEqual(len(pcr_df), 2)
        self.assertEqual(len(good_primers), 1)
        self.assertEqual(len(bad_primers), 1)

    def test_merge_good_total(self):
        """
        Merges pcr df with total primers df.
        """
        self.assertEqual(len(ap.merge_good_total(self.good_primers, self.primer_file)), 12)

    def test_filter_merged(self):
        """
        Drops off target products.
        """
        self.assertEqual(len(ap.filter_merged(self.merged_df, 0)), 0)

    def top_ranked_final_primers(self):
        """
        Selects the top ranking primer pair from each target.
        """
        self.assertEqual(len(ap.top_ranked_final_primers(self.filtered)), 1)

    def test_to_order_plate(self):
        """
        Generates a ready made primer ordering sheet.
        """
        forward, reverse = ap.to_order_plate(self.filtered)
        self.assertEqual(len(forward.columns), 3)
        self.assertEqual(len(forward), 1)

    def tearDown(self):
        pass

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
Unit tester for mp_class_sv.py

Python version: Python 3.6.8 :: Anaconda, Inc.
Dependencies:
 - unittest
 - pandas >= 0.22.0
"""

import unittest
import pandas as pd
from primer_tk.mp_class_sv import MissingPrimers
from primer_tk.mp_class_sv import create_df

class TestMpClassSV(unittest.TestCase):
    """
    Subclass of unittest to test mp_class_sv.py
    """
    def setUp(self):
        self.mp = MissingPrimers("test/data/primer_dump_sv.txt", 1)

    def test_group_seqids(self):
        """
        Asserts proper list is  generated for given number of sample values.
        """
        self.assertEqual(len(self.mp._MissingPrimers__group_seqids()), 3)

    def test_group_seqids_unfilled(self):
        """
        Tests to ensure NA values have not been inserted into primer3 string.
        """
        no_missing_vals = self.mp._MissingPrimers__group_seqids()[1]
        self.assertFalse("NA" in no_missing_vals)

    def test_fill_empty_values(self):
        """
        Tests to ensure that NA vals have been inserted into samples with missing primers.
        """
        all_params = self.mp._MissingPrimers__fill_empty_values()
        missing_params = all_params[1]
        self.assertTrue("=NA" in missing_params[-2])

    def test_gather_primer_info(self):
        """
        Test to ensure that if 1 value is passed, all sample names
        end with 1. Ex: SampleABC1
        """
        primer_info = self.mp._MissingPrimers__gather_primer_info()
        self.assertTrue(all(item[0].endswith('1') for item in primer_info))

    def test_len_list_output(self):
        """
        Tests to ensure the proper number of params have been kept from the p3 output.
        Slightly longer than normal output because coordinate positions are kept for downstream.
        """
        prim_list_1 = self.mp.samp_primer_info
        for sample_data in prim_list_1:
            self.assertEqual(len(sample_data), 10)

    def test_create_df(self):
        """
        Tests to ensure a df is created from primer list.
        """
        prim_list_1 = self.mp.samp_primer_info
        primer_df = create_df([prim_list_1])
        self.assertTrue(isinstance(primer_df, pd.DataFrame))

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

"""
Modules required to run program.
Dependencies:
  - python3.6+
    - pandas>=0.22.0
    - numpy>=1.14.2
    - biopython>=1.70
    - argparse
"""

import sys
import time
import argparse
from Bio import SeqIO
import pandas as pd

def add_iterator_subparser(subparser):
    """
    Get commandline arguments
    Args:
        subparser (?): Subparser object.
    Returns:
        args (Namespace): the parsed arguments.
    """

    parser = subparser.add_parser("iterator", help="Iterator subparser")

    parser.add_argument("-ref", "--ref_genome", required=True,
                        help="Reference Genome File to design primers around")
    parser.add_argument("-in", "--regions_file", required=True,
                        help="File with regions to design primers around")
    parser.add_argument("-opt_size", "--primer_opt_size",
                        dest="primer_opt_size", default="22",
                        help="The optimum primer size for output, default: 22")
    parser.add_argument("-min_size", "--primer_min_size",
                        dest="primer_min_size", default="18",
                        help="The optimum primer size for output, default: 18")
    parser.add_argument("-max_size", "--primer_max_size",
                        dest="primer_max_size", default="25",
                        help="The optimum primer size for output, default: 25")
    parser.add_argument("-opt_gc", "--primer_opt_gc",
                        dest="primer_opt_gc", default="50",
                        help="Optimum primer GC, default: 50")
    parser.add_argument("-min_gc", "--primer_min_gc",
                        dest="primer_min_gc", default="20",
                        help="Minimum primer GC, default: 20")
    parser.add_argument("-max_gc", "--primer_max_gc",
                        dest="primer_max_gc", default="80",
                        help="Maximum primer GC, default: 80")
    parser.add_argument("-opt_tm", "--primer_opt_tm",
                        dest="primer_opt_tm", default="60",
                        help="Optimum primer TM, default: 60")
    parser.add_argument("-min_tm", "--primer_min_tm",
                        dest="primer_min_tm", default="57",
                        help="minimum primer TM, default: 57")
    parser.add_argument("-max_tm", "--primer_max_tm",
                        dest="primer_max_tm", default="63",
                        help="maximum primer TM, default: 63")
    parser.add_argument("-sr", "--product_size_range",
                        dest="product_size_range", default="200-400",
                        help="Size Range for PCR Product, default=200-400")
    parser.add_argument("-flank", "--flanking_region_size",
                        dest="flanking_region_size", default="200",
                        help="This value will select how many bases up and downstream to count\
                              when flanking SNP (will do 200 up and 200 down), default: 200")
    parser.add_argument("-st", "--sequence_target",
                        dest="sequence_target", default="199,1",
                        help="default: 199,1, should be half of your flanking region size,\
                              so SNP/V will be included.")
    parser.add_argument("-mp", "--mispriming", dest="mispriming",
                        help="full path to mispriming library for primer3\
                              (EX: /home/dkennetz/mispriming/humrep.ref")
    parser.add_argument("-tp", "--thermopath", dest="thermopath",
                        help="full path to thermo parameters for primer3 to use\
                              (EX: /home/dkennetz/primer3/src/primer3_config/) install loc")

    return parser

def genome_iterator(genome):
    """
    Uses Biopython SeqIO to parse a genome Fasta and store each chr and
    sequence as a list.

    Args:
        genome (file): Full reference genome from command-line arguments.

    Returns:
        output (list): genome parsed into list of tuples of (header, seq)
        by chromosome for easy access later.
    """
    output = []
    with open(genome, "r") as handle:
        for record in SeqIO.parse(handle, "fasta"):
            chrm = record.id
            seq = record.seq
            output.append((chrm, seq))
    return output

def create_dataframe_csv(regions_file):
    """
    Creates a pandas DataFrame from a regions file in comma separated form
    and names the columns in the DF according to their values.

    Args:
        regions_file (file): Full path to -in input regions.csv.
        This should have coordinate positions of interest to design primers around.
        Note: the chromosome column can be of format chr1 or simply 1 (chr not necessary).
        The file should contain no headers and should be structured as follows:

        Gene1,Sample1,chr1,pos1
        Gene2,Sample2,chr2,pos2
        Gene3,Sample3,chr3,pos3
        ...
    Returns:
        regions_df (pd.DataFrame): The infile parsed to pd.DataFrame object.
    """

    regions_df = pd.read_csv(regions_file, header=None)
    regions_df.columns = ['Gene', 'Sample', 'Chr', 'Pos']
    regions_df = regions_df.astype({'Chr': str})
    return regions_df

def create_dataframe_txt(regions_file):
    """
    Creates a pandas DataFrame from a regions file in a tab separated form
    and uses the following column names in the DF according to their values.

    Args:
        regions_file (file): Full path to -in input regions.txt.
        This should have coordinate positions of interest to design primers around.
        Note: the chromosome column can be of format chr1 or simply 1 (chr not necessary).
        The file should contain no headers and should be constructed as follows:

        Gene1\tSample1\tchr1\tpos1
        Gene2\tSample2\tchr2\tpos2
        Gene3\tSample3\tchr3\tpos3
        ...
    Returns:
        regions_df (pd.DataFrame): The infile parsed to pd.DataFrame object.
    """
    regions_df = pd.read_table(regions_file, header=None)
    regions_df.columns = ['Gene', 'Sample', 'Chr', 'Pos']
    regions_df = regions_df.astype({'Chr': str})
    return regions_df

def file_extension(infile):
    """
    Runs pd.read_ function to create dataframe corresponding
    to file extension, .txt and .csv accepted.
    This will call the function to create a df only if file extension is correct.
    If the file extension is wrong, it will return the error message and exit.

    Args:
        infile (file): Full path to -in input regions.txt.
    Returns:
        small_regions (pd.DataFrame): The infile parsed to pd.DataFrame object
    """
    if infile.lower().endswith('.txt'):
        small_regions = create_dataframe_txt(infile)
    elif infile.lower().endswith('.csv'):
        small_regions = create_dataframe_csv(infile)
    else:
        sys.exit("Wrong File Format, should be .txt or .csv")
    return small_regions

def match_chr_to_genome(dataframe, genome):
    """
    Used to match formatting between regionsFile input and genome for chromosome
    denotations. Some genomes contain the string "chr" and some do not.

    Args:
        dataframe (pandas object): dataframe created from create_dataframe() function.
        genome (list): genome list of tuples created from genome_iterator() function.

    Returns:
        dataframe (pandas object): dataframe formatted to match genome annotation of chromosome.
    """
    if dataframe['Chr'].str.contains("chr").any() and "chr" in str(genome[0][0]):
        pass
    elif dataframe['Chr'].str.contains("chr").any() and "chr" not in str(genome[0][0]):
        dataframe['Chr'] = dataframe['Chr'].str.replace('chr', '')
    elif not dataframe['Chr'].str.contains("chr").any() and "chr" not in str(genome[0][0]):
        pass
    elif not dataframe['Chr'].str.contains("chr").any() and "chr" in str(genome[0][0]):
        dataframe['Chr'] = 'chr' + dataframe['Chr']
    else:
        print("look at pandas error")
    return dataframe

def create_flanking_regions_fasta(genome, dataframe, flanking_region_size):
    """
    Makes batch processing possible, pulls down small
    region of genome for which to design primers around.
    This is based on the chromosome and position of input file.
    Each Fasta record  will contain:
    >Sample_Gene_chr:pos__
    Seq of flanking region

    Args:
        genome (list): genome list of tuples (header, seq).
        dataframe (pandas object): dataframe with sample info.
        flanking_region_size (int): length of sequence upstream and downstream of
        input coordindate position to pull as sequence to design primers around.
    Returns:
        output (list): list of tuples with (header, seq) where seq is flanking region
        and header is sample ID.
    """
    output = []
    for headers, seqs in genome:
        chrm = str(headers)
        seq = str(seqs)
        for gene, sample, chrom, pos in zip(dataframe.Gene, dataframe.Sample,
                                            dataframe.Chr, dataframe.Pos):
            if str(chrom) == chrm:
                header = str(str(sample)+"_"+str(gene)+"_"+str(chrom)+":"+str(pos)+"__")
                flank_seq = seq[int(pos)-int(flanking_region_size):\
                                    int(pos)+int(flanking_region_size)]
                output.append((header, flank_seq.upper()))
    return output

if __name__ == "__main__":
    main()
